*! cmp 6.4.2 21 July 2013
*! Copyright David Roodman 2007-13. May be distributed free.
*! Version history at bottom

cap program drop cmp
program define cmp, sortpreserve properties(user_score svyb svyj svyr mi fvaddcons)
	version 10.0
	cap version 11.0
	
	if replay() {
		if "`e(cmd)'" != "cmp" error 301
		if _by() error 190
		Display `0'
		exit 0
	}

	cmp_clear

	global cmp_out 0
	global cmp_cont 1
	global cmp_left 2
	global cmp_right 3
	global cmp_probit 4
	global cmp_oprobit 5
	global cmp_mprobit 6
	global cmp_int 7
	global cmp_trunc 8 // deprecated
	global cmp_probity1 8 // now used in _cmp_ind* vars to indicate probit obs with y!=0
	global cmp_roprobit 9
	
	if `"`0'"' == "setup" {
		di as txt "$" "cmp_out      = " as res 0
		di as txt "$" "cmp_cont     = " as res 1
		di as txt "$" "cmp_left     = " as res 2
		di as txt "$" "cmp_right    = " as res 3
		di as txt "$" "cmp_probit   = " as res 4
		di as txt "$" "cmp_oprobit  = " as res 5
		di as txt "$" "cmp_mprobit  = " as res 6
		di as txt "$" "cmp_int      = " as res 7
		di as txt "$" "cmp_trunc    = " as res 8 as txt " (deprecated)"
		di as txt "$" "cmp_roprobit = " as res 9
		exit 0
	}

	cap ghk2version
	if _rc | "`r(version)'" < "01.50.00" {
		di as err "Error: {cmd:cmp} works with {cmd:ghk2()} version 1.5.0 or later."
		di `"To install or update it, type or click on {stata "ssc install ghk2, replace"}. Then restart Stata."'
		exit 601
	}

	syntax anything(equalok id="model" name=model) [pw fw aw iw] [if] [in], INDicators(string asis) [svy GHKAnti GHKDraws(string) ///
		GHKType(string) QUIetly noLRtest CLuster(varname) Robust vce(string) SCore(string) Level(real `c(level)') predict lnl(string) ///
		CONSTraints(numlist) TECHnique(string) INTERactive noDRop init(namelist min=1 max=1) lf pseudod2 PSampling(numlist min=1 max=2) ///
		STRUCtural REVerse noESTimate REDraws(string) COVariance(string) INTPoints(string) INTMethod(string) noAUTOconstrain noSIGXform *] 
	if "`pseudod2'" != "" cmp_error 198 "The pseudod2 option is no longer supported."

	local cmdline `0'

	local structural = "`structural'" != ""
	global cmp_reverse = "`reverse'" != ""
	mata _reverse = $cmp_reverse
	global cmpSigXform = "`sigxform'" ==""
	mata _SigXform = $cmpSigXform
	if $cmpSigXform {
		local ln ln
		local atanh atanh
	}

	if c(stata_version) < 11.1 {
		di as res _n "Note: cmp runs faster in Stata versions 11.2 and higher."
		if c(stata_version) >= 11.0 {
			di as res "You should be able to upgrade to 11.2 for free. Type or click on {stata "update executable"}."
		}
	}
	
	_get_eformopts, soptions eformopts(`options')
	local eform `s(eform)'
	local options `s(options)'

	marksample touse
	
	if "`svy'" != "" {
		svymarkout `touse'
		svyopts modopts diopts options, `options' `eform' level(`level')
		local subopt subpop(`s(subpop)')
		local meff `s(meff)'
	}
	else local diopts `eform' level(`level')

	if "`subpop'"=="" local subpop 1
		else markout `touse' `subpop'

	mlopts mlopts, `options'
	local collinear = s(collinear)

	Parse `model' // parse the equations
	global cmp_d $parse_d

	if $parse_L == 1 global cmp_intmethod 0
	else {
		mata _onestep = 0
		if `"`redraws'"'=="" {
			if "`intmethod'"'!="" {
					local 0 `intmethod'
					syntax [anything(name=intmethod)], [ONEstep]
			}
			if "`intmethod'"'=="" local intmethod mvaghermite
			local 0, `intmethod'
			syntax, [Ghermite MVAghermite RZghermite]
			local methods ghermite mvaghermite rzghermite
			local t: list posof "`ghermite'`mvaghermite'`rzghermite'" in methods
			if `t' global cmp_intmethod = `t' - 1
				else cmp_error 198 `"The {cmdab:intm:ethod()} option, if included, should be "ghermite", "mvaghermite", or "rzghermite"."'
			global cmp_onestep = "`onestep'" != ""
			mata _onestep=$cmp_onestep
		}
		else global cmp_intmethod 0
	}

	global parse_wtypeL `weight'
	global parse_wexpL `"`exp'"'
	local 0 `ghkdraws'
	syntax [anything], [type(string) ANTIthetics SCRamble]
	if `"`ghktype'"' != "" & `"`type'"' != "" & `"`ghktype'"' != `"`type'"' & `"`ghktype'`type'"' != "halton" {
		di as res _n "Warning: {cmd:type(`type')} suboption overriding deprecated {cmd:ghktype(`ghktype')} option."
	}
	if `"`type'"' != "" local ghktype `type'
	local 0, ghkdraws(`anything')
	syntax, [ghkdraws(numlist integer>=1 max=1)]
	if `"`ghktype'"'=="" local ghktype halton
	else if inlist(`"`ghktype'"', "halton", "hammersley", "ghalton", "random") == 0 {
		cmp_error 198 `"The {cmdab:ghkt:ype()} option must be "halton", "hammersley", "ghalton", or "random". It corresponds to the {cmd:{it:type}} option of {cmd:ghk()}. See help {help mf_ghk}."'
	}
	if "`scramble'"!="" & "`type'"=="ghalton" {
		di as res "Warning: {cmd:scramble} in {cmd:ghkdraws()} option incompatible with {cmd:ghalton}. {cmd:scramble} ignored."
	}
	if 0`ghkdraws' mata CheckPrime(`ghkdraws')
	mata _ghkType="`ghktype'"; _ghkAnti = `="`antithetics'"!="" | "`ghkanti'"!=""'; _ghkDraws=0`ghkdraws'; _ghkScramble="`scramble'"!=""

	if `"`covariance'"' == "" {
		forvalues l=1/$parse_L {
			global cmp_cov`l' unstructured
		}
	}
	else {
		local covariance: subinstr local covariance "." "unstructured", word all
		if `:word count `covariance'' != $parse_L cmp_error 198 "The {cmdab:cov:ariance()}, if used, must contain one entry for each of the $parse_L levels in the model."
		else {
			tokenize `covariance'
			forvalues l=1/$parse_L {
				local 0, ``l''
				syntax, [UNstructured EXchangeable INDependent]
				global cmp_cov`l' `unstructured'`exchangeable'`independent'
				if inlist("${cmp_cov`l'}", "unstructured", "exchangeable", "independent") == 0 {
					cmp_error 198 `"Each entry in the {cmdab:cov:ariance()} option must be "unstructured", "." (equivalent to "unstructured"), "exchangeable", or "independent"."'
				}
			}
		}
	}
	forvalues l=1/$parse_L {
		local FixedRhoFill`l' = cond("${cmp_cov`l'}"=="independent", 0, .)
	}

	if "`lf'" != "" & $parse_L > 1 & c(stata_version) <= 11.1 cmp_error 198 "The lf option is not available for multi-level models in Stata version 11.1 and lower."
	
	local t : subinstr local indicators "(" "", all
	if $cmp_d != `:word count `:subinstr local t ")" "", all'' cmp_error 198 `"The indicators() option must contain $cmp_d variables, one for each equation. Did you forget to type {stata "cmp setup"}?"'

	mata _Quadrature = 0; _REAnti = 1
	if $parse_L > 1 {
		if `"`redraws'"' == "" {
			if $parse_L > 2 & $cmp_intmethod di as res "Warning: for models with more than 2 levels, adaptive quadrature does not (yet) work well. Try simulation, via the {cmdab:red:raws()} option."
			if `"`intpoints'"' == "" {
				forvalues l=2/$parse_L {
					local intpoints `intpoints' 12 // default precision level for quadrature
				}
				local redraws `intpoints'
			} 
			else {
				local 0, intpoints(`intpoints')
				syntax, [intpoints(numlist integer>=1)]
				if $parse_L!=`:word count `intpoints''+1 cmp_error 198 "If included, the intpoints() should have one entry for each level except the lowest."
				tokenize `intpoints'
				forvalues l=1/`=$parse_L-1' {
					if ``l'' > 25 {
						di as res "Warning: quadrature precision limited to the equivalent of 25 integration points."
						local `l' 25
					}
					local redraws `*'
				}
			}
			local steps 1
			mata _Quadrature = _REAnti = 1
		} 
		else {
			if `"`intpoints'"' != "" cmp_error 198 "intpoints() and redraws() options conflict. Use one or neither. (Default: sparse-grid quadrature with precision equivalent to 12 integration points.)"
			local 0 `redraws'
			syntax [anything], [type(string) ANTIthetics STeps(numlist integer min=1 max=1 >0) SCRamble]
			local 0, redraws(`anything')
			syntax, [redraws(numlist integer>=1)]
			if $parse_L!=`:word count `redraws''+1 cmp_error 198 "If included, the redraws() option should have one entry for each level except the lowest."

			if `"`type'"'=="" local type halton
			else if inlist(`"`type'"', "halton", "hammersley", "ghalton", "random") == 0 {
				cmp_error 198 `"The {cmd:redraws()} {cmd:type()} suboption must be "halton", "hammersley", "ghalton", or "random"."'
			}
			if "`type'"=="hammersley" & "`ghktype'"=="hammersley" {
				cmp_error 198 "Random effects and GHK sequences shouldn't both be Hammersley since this will assign the same draws to the first dimension of each."
			}
			if "`scramble'"!="" & "`type'"=="ghalton" {
				di as res "Warning: {cmd:scramble} in {cmd:redraws()} option incompatible with {cmd:ghalton}. {cmd:scramble} ignored."
			}
			mata CheckPrime(strtoreal(tokens("`redraws'")))
			mata _REType="`type'"; _REAnti = `=1+("`antithetics'"!= "")'; _REScramble = "`scramble'"!=""
		}
	}
	if 0`steps'==0 local steps 1

	global cmp_max_cuts 0
	global cmp_tot_cuts 0
	global cmp_num_mprobit_groups 0
	global cmp_num_roprobit_groups 0
	global cmp_mprobit_ind_base 20
	global cmp_roprobit_ind_base 40
	global cmp_intreg 0
	global cmp_truncreg 0
	global cmp_NSimEff 0
	local asprobit_eq 0
	tempvar _touse n asmprobit_dummy_sum asmprobit_ind
	qui {
		gen double _cmp_lnfi = . in 1
		gen byte `_touse' = 0
		tokenize `"`indicators'"', parse("() ")
		local parse_eqno 0
		local cmp_eqno 0
		while `"`1'"' != "" {
			if (`"`1'"' == ")" & `asprobit_eq' == 0) | (`"`1'"' == "(" & `asprobit_eq') cmp_error 132 "Too many `1'"
			if `"`1'"'==")" {
				if "`mro'" == "m" {
					count if `asmprobit_dummy_sum'!=1 & `touse' & _cmp_ind`first_asprobit_eq'
					if r(N) cmp_error 132 "For multinomial probit groups, exactly one dependent variable must be non-zero for each observation."
					replace _cmp_ind`first_asprobit_eq'=`asmprobit_ind'*(_cmp_ind`first_asprobit_eq'!=0) // store choice info in indicator var for first equation
					drop `asmprobit_ind' `asmprobit_dummy_sum'
				}
				mat cmp_`mro'probit_group_inds[${cmp_num_`mro'probit_groups}, 2] = `cmp_eqno'
				mat cmp_nonbase_cases = nullmat(cmp_nonbase_cases) , 0 , J(1, `asprobit_eq'-2, 1)
				local asprobit_eq 0
				local mro
				macro shift
				continue
			}

			local ++parse_eqno
			local ++cmp_eqno
			local num_alts 0

			if `"`1'"'=="(" {
				macro shift
				if `"`1'"' == ")" continue
				local asprobit_eq 1
				local first_asprobit_eq `cmp_eqno'
				if "${parse_x`parse_eqno'}" == "" { // assure nocons for first eq in asprobit group or leave cons in but constrained to 0
					constraint free
					local _constraints `_constraints' `r(free)'
					constraint `r(free)' [${parse_eq`parse_eqno'}]_cons
				}
				else global parse_xc`parse_eqno' nocons			
			}

			gen byte _cmp_ind`cmp_eqno' = `1'
			count if inlist(_cmp_ind`cmp_eqno', 0, 1, 2, 3, 4, 5, 6, 7, 8, 9)==0 & _cmp_ind`cmp_eqno' < . & `touse'
			if r(N) cmp_error 198 "Indicator for ${parse_y`parse_eqno'} must only evaluate to integers between 0 and 9."

			replace _cmp_ind`cmp_eqno' = $cmp_cont if _cmp_ind`cmp_eqno'==$cmp_trunc // deprecate this indicator value
			
			foreach macro in eq x xc xo xe yR {
				global cmp_`macro'`cmp_eqno' ${parse_`macro'`parse_eqno'}
			}
			global cmp_y`cmp_eqno' ${parse_y`parse_eqno'}

			markout _cmp_ind`cmp_eqno' ${cmp_xo`cmp_eqno'} ${cmp_xe`cmp_eqno'}
			if "${cmp_xe`cmp_eqno'}" != "" replace _cmp_ind`cmp_eqno' = 0 if ${cmp_xe`cmp_eqno'}<=0
			
			count if _cmp_ind`cmp_eqno'==$cmp_int & `touse'
			if r(N) { // interval regression
				global cmp_y`cmp_eqno'_L : word 1 of ${cmp_y`cmp_eqno'}
				global cmp_y`cmp_eqno'   : word 2 of ${cmp_y`cmp_eqno'}
				if `"${cmp_y`cmp_eqno'}"' == "" cmp_error 198 "Interval regressions equations require two dependent variables."
				global cmp_intreg 1
				global cmp_intreg`cmp_eqno' 1
				mat cmp_intregeqs = nullmat(cmp_intregeqs), 1
				replace _cmp_ind`cmp_eqno' = ${cmp_y`cmp_eqno'}<. if _cmp_ind`cmp_eqno'==$cmp_int & ${cmp_y`cmp_eqno'}==${cmp_y`cmp_eqno'_L}
				replace _cmp_ind`cmp_eqno' = 0 if _cmp_ind`cmp_eqno'==$cmp_int & ${cmp_y`cmp_eqno'}<${cmp_y`cmp_eqno'_L} & ${cmp_y`cmp_eqno'_L}<.
				markout _cmp_ind`cmp_eqno' ${parse_x`parse_eqno'}
			}
			else {
				mat cmp_intregeqs = nullmat(cmp_intregeqs), 0
				markout _cmp_ind`cmp_eqno' ${parse_x`parse_eqno'} ${parse_y`parse_eqno'}
				global cmp_intreg`cmp_eqno' 0
				global cmp_y`cmp_eqno'_L . // just to prevent syntax errors
			}

			if `"${parse_tr`parse_eqno'}"' != "" { // truncated regression
				gen double _cmp_Lt`cmp_eqno' = `:word 1 of ${parse_tr`parse_eqno'}'
				gen double _cmp_Ut`cmp_eqno' = `:word 2 of ${parse_tr`parse_eqno'}'
				mat cmp_trunceqs = nullmat(cmp_trunceqs), 1
				global cmp_Lt`cmp_eqno' _cmp_Lt`cmp_eqno'
				global cmp_Ut`cmp_eqno' _cmp_Ut`cmp_eqno'
				global cmp_truncreg 1
				global cmp_truncreg`cmp_eqno' 1
				count if _cmp_ind`cmp_eqno' & `touse'
				local N = r(N)
				replace _cmp_ind`cmp_eqno' = 0 if `touse' & inlist(_cmp_ind`cmp_eqno', $cmp_cont, $cmp_left, $cmp_right) & ///
					((${cmp_y`cmp_eqno'}<=${cmp_Lt`cmp_eqno'} & ${cmp_Lt`cmp_eqno'}<.) | ${cmp_y`cmp_eqno'}>=${cmp_Ut`cmp_eqno'})
				replace _cmp_ind`cmp_eqno' = 0 if `touse' & _cmp_ind`cmp_eqno'==$cmp_trunc & ///
					((${cmp_y`cmp_eqno'}<=${cmp_Lt`cmp_eqno'} & ${cmp_Lt`cmp_eqno'}<.) | ${cmp_y`cmp_eqno'_L}>=${cmp_Ut`cmp_eqno'})
				replace _cmp_ind`cmp_eqno' = 0 if `touse' & _cmp_ind`cmp_eqno'==$cmp_probit & ///
					((0<${cmp_Lt`cmp_eqno'} & ${cmp_Lt`cmp_eqno'}<.) | 0>${cmp_Ut`cmp_eqno'}) // truncation range must embrace 0 for probits
				count if _cmp_ind`cmp_eqno' & `touse'
				if r(sum)!=`N' di as res `N'-r(sum) " observations dropped because dependent variable in equation `parse_eqno' is outside truncation range."

				if ${cmp_intreg`cmp_eqno'} {
					gen double _cmp_y`cmp_eqno'_L = ${cmp_y`cmp_eqno'_L}
					gen double _cmp_y`cmp_eqno'   = ${cmp_y`cmp_eqno'}
					global cmp_y`cmp_eqno'_L _cmp_y`cmp_eqno'_L
					global cmp_y`cmp_eqno'   _cmp_y`cmp_eqno'
					replace _cmp_y`cmp_eqno'_L = _cmp_Lt`cmp_eqno' if `touse' & _cmp_ind`cmp_eqno'==$cmp_int & (_cmp_y`cmp_eqno'_L==. | (_cmp_y`cmp_eqno'_L<_cmp_Lt`cmp_eqno' & _cmp_Lt`cmp_eqno'<.))
					replace _cmp_y`cmp_eqno'   = _cmp_Ut`cmp_eqno' if `touse' & _cmp_ind`cmp_eqno'==$cmp_int & _cmp_y`cmp_eqno'  >_cmp_Ut`cmp_eqno'
				}
			}
			else {
				global cmp_truncreg`cmp_eqno' 0
				mat cmp_trunceqs = nullmat(cmp_trunceqs), 0
			}

			global cmp_eq`cmp_eqno' = cond("${parse_eq`parse_eqno'}"=="eq`parse_eqno'", subinstr("${cmp_y`cmp_eqno'}", ".", "", .), "${parse_eq`parse_eqno'}")
			if "`:list eqnames & global(cmp_eq`cmp_eqno')'" != "" global cmp_eq`cmp_eqno' `=substr("${cmp_eq`cmp_eqno'}",1,29)'`cmp_eqno'
			local eqnames `eqnames' ${cmp_eq`cmp_eqno'}

			replace `_touse' = `_touse' | _cmp_ind`cmp_eqno'

			count if _cmp_ind`cmp_eqno'==$cmp_oprobit & `touse'
			if r(N) { // ordered probit
				tempvar t
				// copy in order to get rid of any non-numeric value labels, which can't be stored in e(cat)
				gen `:type ${cmp_y`cmp_eqno'}' `t' = ${cmp_y`cmp_eqno'} if `touse' & _cmp_ind`cmp_eqno'==$cmp_oprobit 
				egen int _cmp_y`cmp_eqno' = group(`t') if `t' < ., lname(cmp_y`cmp_eqno'_label)
				drop `t'
				sum _cmp_y`cmp_eqno' if `touse' & _cmp_ind`cmp_eqno'==$cmp_oprobit, meanonly
				local t = r(max) - 1
				mat cmp_num_cuts = nullmat(cmp_num_cuts) \ `t'
				if $cmp_max_cuts < `t' global cmp_max_cuts = `t'
				global cmp_tot_cuts = $cmp_tot_cuts + `t'
				forvalues j=1/`t' {
					local cutparams `cutparams' /cut_`cmp_eqno'_`j'
				}
				global cmp_y`cmp_eqno' _cmp_y`cmp_eqno'
				local i_oprobit_ys `i_oprobit_ys' i._cmp_y`cmp_eqno'
			}
			else {
				local lrtest `lrtest' ${cmp_xc`cmp_eqno'}
				mat cmp_num_cuts = nullmat(cmp_num_cuts) \ 0
			}
	
			count if _cmp_ind`cmp_eqno'==$cmp_mprobit & `touse'
			local N_mprobit `r(N)'
			count if _cmp_ind`cmp_eqno'==$cmp_roprobit & `touse'
			local N_roprobit `r(N)'
			if `N_mprobit' | `N_roprobit' { // multinomial or rank-ordered probit
				if (`N_mprobit' & "`mro'" == "ro") | (`N_roprobit' & "`mro'" == "m") cmp_error 148 "Cannot mix multinomial and rank-ordered indicator values in the same group."

				count if inlist(_cmp_ind`cmp_eqno', 0, $cmp_mprobit, $cmp_roprobit)==0 & `touse'
				if r(N) | `N_mprobit'&`N_roprobit' cmp_error "Dependent variables modeled as `=cond(`N_mprobit',"multinomial","rank-ordered")' probit may not be modeled differently for other observations in the same equation."
				
				if ${cmp_truncreg`cmp_eqno'} cmp_error "Truncation not allowed in `=cond(`N_mprobit',"multinomial","rank-ordered")' probit equations."
				
				if `asprobit_eq'==1 & "`mro'" == "" { // starting new asprobit group?
					if `N_mprobit' {
						gen byte `asmprobit_dummy_sum' = 0 if `touse'
						gen byte `asmprobit_ind' = $cmp_mprobit_ind_base + `cmp_eqno' - 1 if `touse'
						local mro m
					}
					else local mro ro
					
					global cmp_num_`mro'probit_groups = ${cmp_num_`mro'probit_groups} + 1
					mat cmp_`mro'probit_group_inds = nullmat(cmp_`mro'probit_group_inds) \ (`cmp_eqno', .)
				}
				
				if `asprobit_eq' == 0 { // non-as mprobit?
					if `N_roprobit' cmp_error 148 "Rank-ordered probit indicators must be grouped in parentheses."

					global cmp_num_mprobit_groups = $cmp_num_mprobit_groups + 1

					egen int _cmp_y`cmp_eqno' = group(${cmp_y`cmp_eqno'}) if `touse' & _cmp_ind`cmp_eqno'==$cmp_mprobit, lname(cmp_y`cmp_eqno'_label)

					sum _cmp_y`cmp_eqno', meanonly
					local num_alts = r(max) - 1
					if `num_alts' == 0 cmp_error 148 "There is only one outcome in ${cmp_y`cmp_eqno'}."

					mat cmp_mprobit_group_inds = nullmat(cmp_mprobit_group_inds) \ (`cmp_eqno', `cmp_eqno'+`num_alts')
					mat cmp_nonbase_cases = nullmat(cmp_nonbase_cases) , 0 , J(1, `num_alts', 1)

					// indicator for first equation holds choice info
					replace _cmp_ind`cmp_eqno' = $cmp_mprobit_ind_base + _cmp_y`cmp_eqno' + `cmp_eqno' - 1 if `touse' & _cmp_ind`cmp_eqno'==$cmp_mprobit

					xi, noomit prefix(_mp) i._cmp_y`cmp_eqno'
					forvalues i=`cmp_eqno'/`=`cmp_eqno'+`num_alts'' {
						ren _mp_cmp_y`cmp_eqno'_`=`i'+1-`cmp_eqno'' _mp_cmp_y`i'
						global cmp_y`i' _mp_cmp_y`i'
					}

					LabelMprobitEq `cmp_eqno' `cmp_eqno' `parse_eqno' 1

					forvalues j=`=`cmp_eqno'+1'/`=`cmp_eqno'+`num_alts'' { // Generate all equations associated with this, the user's one mprobit equation
						gen byte _cmp_ind`j' = $cmp_mprobit*(_cmp_ind`cmp_eqno'>0) if `touse'
						LabelMprobitEq `j' `cmp_eqno' `parse_eqno' `j'
						foreach macro in x xc xo xe yR {
							global cmp_`macro'`j' ${cmp_`macro'`cmp_eqno'}
						}
						mat cmp_num_cuts = cmp_num_cuts \ 0
					}
					label drop cmp_y`cmp_eqno'_label

					// first equation in expanded group is placeholder, for consistency with asmprobit--constant-only and constant=0
					constraint free
					local _constraints `_constraints' `r(free)'
					constraint `r(free)' [${cmp_eq`cmp_eqno'}]_cons
					foreach macro in x xc xo xe yR {
						global cmp_`macro'`cmp_eqno' 
					}
				}
				else {
					if "`mro'" == "m" {
						replace `asmprobit_ind' = `asmprobit_ind' + (${cmp_y`cmp_eqno'}!=0) * `asprobit_eq' if _cmp_ind`cmp_eqno'
						replace `asmprobit_dummy_sum' = `asmprobit_dummy_sum' + (${cmp_y`cmp_eqno'}!=0) if _cmp_ind`cmp_eqno'
						replace _cmp_ind`cmp_eqno' = 0 if _cmp_ind`first_asprobit_eq' == 0  // exclude obs missing for base case
					}
					else {
						count if `touse' & (mod(${cmp_y`cmp_eqno'}, 1) | ${cmp_y`cmp_eqno'}<0 | ${cmp_y`cmp_eqno'}>`=maxbyte()-$cmp_roprobit_ind_base')
						if r(N) cmp_error 148 "Dependent variables modeled as rank-ordered probit must take integer values between 0 and `=maxbyte()-$cmp_roprobit_ind_base'."
						replace _cmp_ind`cmp_eqno' = ${cmp_y`cmp_eqno'} + $cmp_roprobit_ind_base if _cmp_ind`cmp_eqno'
					}
					local ++asprobit_eq
				}
			}
			else {
				if `asprobit_eq' {
					if "`mro'"=="m" cmp_error 148 "Each indicator in an alternative-specific multinomial probit group must evaluate to $cmp_mprobit ($"  "cmp_mprobit) at least once."
					           else cmp_error 148 "Each indicator in a rank-ordered probit group must evaluate to $cmp_roprobit ($" "cmp_roprobit) at least once."
				}
				mat cmp_nonbase_cases = nullmat(cmp_nonbase_cases) , 1
			}

			forvalues i=`cmp_eqno'/`=`cmp_eqno'+`num_alts'*(`asprobit_eq'==0)' { // do once unless expanding non-as mprobit eq
				if `i'==1 mat cmp_fixed_rhos$parse_L = 0
				else      mat cmp_fixed_rhos$parse_L = (cmp_fixed_rhos$parse_L, J(`i'-1, 1, .)) \ J(1, `i', `FixedRhoFill$parse_L')

				// create sig param unless mprobit eq 1-2 or (ordered) probit
				count if inlist(_cmp_ind`i', $cmp_probit, $cmp_oprobit) & `touse'
				if r(N) {
					mat cmp_fixed_sigs$parse_L = nullmat(cmp_fixed_sigs$parse_L), 1
					forvalues l=1/`=$parse_L-1' {
						mat cmp_fixed_sigs`l' = nullmat(cmp_fixed_sigs`l'), .
					}
					count if !inlist(_cmp_ind`i', 0, $cmp_oprobit) & `touse'
					if r(N) == 0 global cmp_xc`i' nocons
				}
				else if `asprobit_eq'-1==1 | (`num_alts' & `i'==`cmp_eqno') { // 1st eq of m/roprobit. sig=1 for structural, 0 otherwise, all levels
					forvalues l=1/$parse_L {
						mat cmp_fixed_sigs`l' = nullmat(cmp_fixed_sigs`l'), `structural'
					}
				}
				else if `asprobit_eq'-1==2 | (`num_alts' & `i'==`cmp_eqno'+1) { // 2nd eq of m/roprobit. sig=1 for structural, 2 otherwise, bottom level
					mat cmp_fixed_sigs$parse_L = nullmat(cmp_fixed_sigs$parse_L), sqrt(2-`structural')
					forvalues l=1/`=$parse_L-1' {
						mat cmp_fixed_sigs`l' = nullmat(cmp_fixed_sigs`l'), .
					}
				}
				else {
					forvalues l=1/`=$parse_L-1' {
						mat cmp_fixed_sigs`l' = nullmat(cmp_fixed_sigs`l'), .
					}
					if `i'>=`cmp_eqno'+2 & "${parse_iia`parse_eqno'}" != "" { // impose IIA for non-as mprobits
						mat cmp_fixed_sigs$parse_L = nullmat(cmp_fixed_sigs$parse_L), sqrt(2-`structural')
						forvalues j=`=`cmp_eqno'+1'/`=`i'-1' {
							mat cmp_fixed_rhos$parse_L[`i',`j'] = cond(`structural', 0, cond($cmpSigXform, atanh(.5), .5))
						}
					}
					else {
						if "${cmp_cov$parse_L}" != "exchangeable" local sigparams$parse_L `sigparams$parse_L' /`ln'sig_`i' 
						mat cmp_fixed_sigs$parse_L = nullmat(cmp_fixed_sigs$parse_L), .
					}
				}

				global cmp_id`i' ${parse_id`parse_eqno'}

				// split probit samples into y==0 and y!=0 parts for efficient computation of likelihood
				replace _cmp_ind`i' = cond(_cmp_ind`i'==$cmp_probit & ${cmp_y`i'}, $cmp_probity1, _cmp_ind`i') if `touse' // to streamline likelihood computation split probit samples into y=0 and y!=0

				forvalues l=1/`=$parse_L-1' {
					if cmp_fixed_sigs`l'[1,`i'] { // suppress RC and RE lists if this is a base case without structural, so sigs=0
						global cmp_rc`l'_`i' ${parse_rc`l'_`parse_eqno'}
						global cmp_re`l'_`i' ${parse_re`l'_`parse_eqno'}
						global cmp_cov`l'_`i' ${parse_cov`l'_`parse_eqno'}
					}
					local cmp_NumEff`l'_`i': word count ${cmp_rc`l'_`i'} ${cmp_re`l'_`i'}
					global cmp_NSimEff = $cmp_NSimEff + `cmp_NumEff`l'_`i''
				}
				global cmp_cov${parse_L}_`i' unstructured
				local  cmp_NumEff${parse_L}_`i' 1  // one "random effect" at bottom
				global cmp_intreg`i' ${cmp_intreg`cmp_eqno'}
				global cmp_truncreg`i' ${cmp_truncreg`cmp_eqno'}
				global cmp_Ut`i' ${cmp_Ut`cmp_eqno'}
				global cmp_Lt`i' ${cmp_Lt`cmp_eqno'}
				if `i'>`cmp_eqno' {
					mat cmp_trunceqs = nullmat(cmp_trunceqs), 0
					mat cmp_intregeqs = nullmat(cmp_intregeqs), 0
				}
			}

			if `asprobit_eq'==0 local cmp_eqno = `cmp_eqno' + `num_alts'
			macro shift
		}

		replace `touse' = `touse' & `_touse'
		markout `touse' _cmp_ind*

		if "`lnl'" != "" gen double `lnl' = . in 1

		global cmp_d `cmp_eqno'
		forvalues eq=1/$cmp_d {
			global cmp_eq $cmp_eq ${cmp_eq`eq'}
		}
		
		global cmpHasGamma 0
		tempname Gamma GammaINobs GammaI GammaId
		mat `GammaI'     = I($cmp_d)
		mat `GammaINobs' = I($cmp_d)
		forvalues eq1=1/$cmp_d {
			foreach EndogVar in ${cmp_yR`eq1'} {
				local eq2: list posof `"`EndogVar'"' in global(cmp_eq)
				if `eq2' {
					mat cmpGammaInd = nullmat(cmpGammaInd) \ `eq1',`eq2'
					mat `GammaI'[`eq1', `eq2'] = 1
					qui count if _cmp_ind`eq2'>1 & `touse'  // is the linear functional referred to sometimes unobserved?
					mat `GammaINobs'[`eq1', `eq2'] = r(N)>0
					global cmp_gammaparams ${cmp_gammaparams} /gamma`eq1'_`eq2'
					global cmpHasGamma = $cmpHasGamma + 1
				}
				else cmp_error 111 `"Equation `EndogVar' not found."'
			}
		}
		mat cmpGammaInd = nullmat(cmpGammaInd) \ .,.
		mata __GammaInds = J($cmp_d, 1, &J(0,1,0))
		if $cmpHasGamma {
			mat `GammaId' = `GammaINobs'
			forvalues eq1=1/`=$cmp_d-2' {
				mat `GammaId' = `GammaId' * `GammaINobs'
			}

			forvalues eq1=1/$cmp_d {
				mata __GammaInds[`eq1'] = &st_matrix("cmpGammaInd")[OneInds(st_matrix("cmpGammaInd")[,1]:==`eq1'),2]
				forvalues eq2=1/$cmp_d {
					if `eq1' != `eq2' & `GammaId'[`eq1',`eq2'] {
						count if _cmp_ind`eq1' & _cmp_ind`eq2'==0
						replace _cmp_ind`eq1' = 0 if _cmp_ind`eq1' & _cmp_ind`eq2'==0
						if r(N) noi di as txt "(" r(N) plural(r(N)," observation") " dropped from ${cmp_eq`eq1'} equation because " cond(r(N)>1,"they are","it is") " unavailable in the ${cmp_eq`eq2'} equation, on which the ${cmp_eq`eq1'} equation depends)"
					}
				}
			}

			mat `GammaId' = `GammaI'
			forvalues eq1=1/`=$cmp_d-2' {
				mat `GammaId' = `GammaId' * `GammaI'
			}
		}
	}

	xi, prefix(" ") noomit `i_oprobit_ys'

	mat cmpEqs = J($cmp_d, $parse_L, 0)
	forvalues eq = 1/$cmp_d {
		foreach id in ${cmp_id`eq'} {
			local l: list posof "`id'" in global(parse_id)
			mat cmpEqs[`eq', `l'] = "`id'"=="_n" | cmp_fixed_sigs`l'[1,`eq']>0  // don't simulate REs with variance=0
		}
	}
	mata _Eqs=J($parse_L, 1, NULL); for (l=$parse_L; l; l--) _Eqs[l]=&OneInds(st_matrix("cmpEqs")[,l]'); _GammaEqs = _Eqs
	if $cmpHasGamma mata for (l=$parse_L; l; l--) _GammaEqs[l]=&OneInds((st_matrix("`GammaId'")*st_matrix("cmpEqs")[,l])':!=0)

	mat cmp_NumEff = J($parse_L, $cmp_d, 0)
	forvalues l=1/$parse_L {
		forvalues eq=1/$cmp_d {
			mat cmp_NumEff[`l', `eq'] = `cmp_NumEff`l'_`eq''
		}
	}
	mata _NumEff=st_matrix("cmp_NumEff")
	
	local technique technique(`technique')
	_vce_parse, optlist(robust jackknife bootstrap oim opg) argoptlist(cluster) pwallowed(robust jackknife bootstrap cluster oim opg) old: `wgtexp', `robust' cluster(`cluster') vce(`vce')
	local vce `r(vceopt)'
	local robust `r(robust)'
	local cluster `r(cluster)'
	markout `touse' `cluster', strok

	tokenize $parse_id
	forvalues l = 1/`=$parse_L-1' {
		mat cmp_fixed_rhos`l' = J($cmp_d, $cmp_d, `FixedRhoFill`l'')
		local ids `ids' ``l''
		qui egen long _cmp_id`l' = group(`ids') if `touse'
		forvalues j=1/$cmp_d {
			global cmp_rc`l' ${cmp_rc`l'} ${cmp_rc`l'_`j'} ${cmp_re`l'_`j'}
		}
		global cmp_rcu`l': list uniq global(cmp_rc`l')
	}

	if $parse_L == 1 { // for 1-level models, ml will handle weights
		if `"$parse_wexpL"' != "" {
			tempvar wvar
			qui gen double `wvar' $parse_wexpL if `touse'
			local wgtexp [$parse_wtypeL = `wvar']
			local awgtexp [aw = `wvar']
			markout `touse' `wvar'
		}
	}
	else {
		sort _cmp_id*

		global parse_wtype$parse_L $parse_wtypeL
		global parse_wexp$parse_L `"$parse_wexpL"'
		forvalues l = 1/$parse_L {
			local cmp_ids `cmp_ids' _cmp_id`l'
			if "${parse_wtype`l'}" != "" {
				qui gen double _cmp_weight`l' ${parse_wexp`l'} if `touse'
				markout `touse' _cmp_weight`l'
				replace `touse' = 0 if _cmp_weight`l'<=0

				if "${parse_wtype`l'}" == "fweight" {
					qui count if trunc(_cmp_weight`l') != _cmp_weight`l' & `touse'
					if r(N) cmp_error 401 "Frequency weights must be integers."
				}
					
				if "${parse_wtype`l'}" == "pweight" & 0`wcluster' == 0 {
					local wcluster 1
					if "`cluster'" != "`:word `l' of $parse_id'" {
						if "`cluster'`robust'" != "" {
							di as res _n "Warning: " as txt `"[pweight ${parse_wexp`l'}]"' as res " would usually imply " as txt "vce(cluster `:word `l' of $parse_id')."
							di as res "Implementing " as txt `"`=cond("`cluster'"=="", "robust", "cluster `cluster'")'"' as res " instead."
						}
						else {
							local vce vce(cluster `:word `l' of $parse_id')
							local lrtest pweight
							di as res _n "Note: " as txt `"[pweight ${parse_wexp`l'}]"' as res " implies vce(cluster `:word `l' of $parse_id')"
						}
					}
				}

				if `l' < $parse_L {
					tempvar t
					qui by `cmp_ids': egen float `t' = sd(_cmp_weight`l') if `touse'
					qui count if `t' & `t'<. & `touse'
					if r(N) cmp_error 101 "Weights for level {res}`:word `l' of $parse_id' {err}must be constant within groups."
					drop `t'
				}
				mata _Weights = 1
			}
		}

		tokenize $parse_id // rebuild id's in case weights<=0 nix some groups
		local ids
		qui forvalues l = 1/`=$parse_L-1' {
			local ids `ids' ``l''
			drop _cmp_id`l'
			egen long _cmp_id`l' = group(`ids') if `touse'
		}
		sort _cmp_id*
	}

	qui count if `touse'
	if r(N)==0 cmp_error 2000 "No observations."
	
	local method_spec = cond("`lf'"!="",  ///
								 cond(c(stata_version) <= 11.1, "lf cmp_lf", "lf0 cmp_lf1"),    ///
								 cond(c(stata_version) <= 11.1, "d1 cmp_d1", "lf1 cmp_lf1"))

	if "`predict'" != "" {
		forvalues l=1/$parse_L {
			mat cmp_fixed_rhos`l' = e(fixed_rhos`l')
			mat cmpSigScoreInds`l' = e(sig_score_inds`l')
		}
		global cmp_num_scores = e(num_scores)
		mata _first_call = 1; _GammaInds = __GammaInds; _NumREDraws = 1 \ strtoreal(tokens("`redraws'"))' * _REAnti
	}

	mata _num_mprobit_groups  = $cmp_num_mprobit_groups;   _mprobit_group_inds=st_matrix("cmp_mprobit_group_inds" ); _mprobit_ind_base  = $cmp_mprobit_ind_base
	mata _num_roprobit_groups = $cmp_num_roprobit_groups; _roprobit_group_inds=st_matrix("cmp_roprobit_group_inds"); _roprobit_ind_base = $cmp_roprobit_ind_base
	mata _nonbase_cases = st_matrix("cmp_nonbase_cases"); _L = $parse_L
	mata _NumCuts = sum(_vNumCuts=st_matrix("cmp_num_cuts")); _intreg = $cmp_intreg; _trunc = $cmp_truncreg; _trunceqs = st_matrix("cmp_trunceqs"); _intregeqs = st_matrix("cmp_intregeqs")
	mata _interactive=`="`interactive'`predict'"!=""'
	mata _HasGamma = $cmpHasGamma

// /lnsigEx_[lev] accross (ergo within too), exchangeable
// /lnsigEx accross, bottom
// /lnsigEx_[lev]_[eq] within-equation, exchangeable
// /lnsig_[coef]_[lev]_[eq] unstructured
// /lnsig_[eq] bottom level
	if "${cmp_cov$parse_L}" == "exchangeable" {
		local sigparams$parse_L `sigparams$parse_L' /`ln'sigEx
	}
	forvalues l=1/`=$parse_L-1' {
		if "${cmp_cov`l'}" == "exchangeable" {
			local sigparams`l' `sigparams`l'' /`ln'sigEx_`l'
		}
		else {
			forvalues eq=1/$cmp_d {
				if cmp_nonbase_cases[1,`eq'] { 
					if "${cmp_cov`l'_`eq'}" == "exchangeable" {
						local sigparams`l' `sigparams`l'' /`ln'sigEx_`l'_`eq'
					}
					else {
						forvalues c=1/`:word count ${cmp_rc`l'_`eq'}' {
												   local sigparams`l' `sigparams`l'' /`ln'sig_`c'_`l'_`eq'
						}
						if "${cmp_re`l'_`eq'}"!="" local sigparams`l' `sigparams`l'' /`ln'sig_`l'_`eq'
					}
				}
			}
		}
	}

// /atanhrhoEx_[lev] accross, exchangeable
// /atanhrhoEx accross, exchangeable, bottom
// /atanhrhoEx_[lev]_[eq] within-equation, exchangeable
// /atanhrho_[coef1]_[coef2]_[lev]_[eq] within, unstructured
// /atanhrho_[eq1][eq2] across, bottom level
// /atanhrho_[lev]_[eq1][eq2] across, upper levels, REs on both sides
// /atanhrho_[coef1]_[coef2]_[lev]_[eq1][eq2] across, upper levels, REs not on both sides
	forvalues l=$parse_L(-1)1 {
		if "${cmp_cov`l'}" == "exchangeable" {
			local sigparams`l' `sigparams`l'' /`atanh'rhoEx`=cond(`l'<$parse_L,"_`l'","")'
		}
		forvalues eq1=1/$cmp_d {
			if "${cmp_cov`l'_`eq1'}"=="exchangeable" & cmp_NumEff[`l', `eq1'] & cmp_nonbase_cases[1,`eq1'] {
				local sigparams`l' `sigparams`l'' /`atanh'rhoEx_`l'_`eq1'
			}
			forvalues c1=1/`=cmp_NumEff[`l', `eq1']' {
				if "${cmp_cov`l'_`eq1'}"=="unstructured" {
					forvalues c2=`=`c1'+1'/`=cmp_NumEff[`l', `eq1']' {
						local sigparams`l' `sigparams`l'' /`atanh'rho_`c1'_`c2'_`l'_`eq1'
					}
				}
				forvalues eq2=`=`eq1'+1'/$cmp_d {
					if cmp_nonbase_cases[1,`eq1'] & cmp_nonbase_cases[1,`eq2'] {
						if cmp_fixed_rhos`l'[`eq2',`eq1']==. &  "${cmp_cov`l'}" == "unstructured" {
							forvalues c2=1/`=cmp_NumEff[`l', `eq2']' {
								local sigparams`l' `sigparams`l'' /`atanh'rho`=cond("`:word `c1' of ${cmp_rc`l'_`eq1'}'`:word `c2' of ${cmp_rc`l'_`eq2'}'"=="","","_`c1'_`c2'")'`_l'_`eq1'`eq2'
							}
						}
					}
					else mat cmp_fixed_rhos`l'[`eq2',`eq1'] = 0
				}
			}
		}
		local _l _`=`l'-1'
	}
	forvalues l=1/$parse_L {
		local sigparams `sigparams' `sigparams`l''
	}
	local auxparams `cutparams' `sigparams'

	if "`predict'" != "" exit  // done reconstructing _cmp_* variables to carry out predict (of scores)

	tempname b cmpInitFull
	// Fit individual models before mis-specifed and constant-only ones in case perfect probit predictors shrink some eq samples
	// Do InitSearch even if user specifies init() to check for that and to build fully labelled parameter vector for constraint work in Estimate
	if "`init'" == "" di as res _n "Fitting individual models as starting point for full model fit."
	`quietly' DoInitSearch InitSearch if `touse' `wgtexp', `svy' adjustprobitsample `drop' auxparams(`auxparams') `=cond("`init'" == "", "", "quietly")'
	mat `cmpInitFull' = r(b)
	local ParamsDisplay `r(ParamsDisplay)'
	local XVarsAll `r(XVarsAll)'

	if $cmpSigXform==0 {
		foreach param in `r(auxparams)' {
			if      substr("`param'", 1, 4)=="/sig" local bounds `bounds' `=substr("`param'", 2, .)'  1e-10 .
			else if substr("`param'", 1, 4)=="/rho" local bounds `bounds' `=substr("`param'", 2, .)' -1 1
		}
	}

	if "`estimate'" != "" {
		NoEstimate `cmpInitFull' `wgtexp'
		ereturn display
		cmp_clear
		di as res _n `"Full model not fit. To view the initial coefficient matrix, type or click on {stata "mat list e(b)"}."'
		di as res "You can copy and modify this matrix, then pass it back to {cmd:cmp} with the {cmd:init()} option."
		exit 0
	}

	local initconstraints `r(initconstraints)'
	local auxparams `r(auxparams)'
	global cmp_num_scores = $cmpHasGamma + $cmp_d + `:word count `auxparams''
	mata _NScores = $cmp_num_scores

	tempvar t
	egen `t' = anycount(_cmp_ind*), values(0)
	qui replace `touse' = 0 if `t'==$cmp_d
	drop `t'

	if $parse_L>1 | "`method_spec'"=="lf cmp_lf" {
		tempname cmpN
		global cmpN `cmpN'
		qui gen long `t' = _n
		sum `t' if `touse', meanonly
		drop `t'
		global cmpObs1 = r(min)
		scalar `cmpN' = r(N)
	}

	forvalues i=1/$cmp_d { // save these for full fit in case they get modified by 1only or meff calls to InitSearch
		local cmp_x`i' ${cmp_x`i'}
		local cmp_xc`i' ${cmp_xc`i'}
	}

	if "`meff'" != "" {
		di as res _n "Fitting misspecified model."
		qui InitSearch if `touse' & (`subpop'), `drop' auxparams(`auxparams')
		mat `b' = r(b)
		Estimate `method_spec' if `touse' & (`subpop'), init(`init') cmpinit(`b') `vce' auxparams(`auxparams') psampling(`psampling') resteps(`steps') bounds(`bounds') ///
			constraints(`constraints' `_constraints' `initconstraints') `autoconstrain' `mlopts' `technique' `quietly' redraws(`redraws') paramsdisplay(`r(ParamsDisplay)')
		if _rc==0 {
			tempname vsmp
			mat `vsmp' = e(V)
		}
	}

	if "`lrtest'" == "" & `"`constraints'`robust'`cluster'`estimate'"'=="" & "`weight'"!="pweight" & "$parse_x"!="" {
		local HasGamma $cmpHasGamma
		global cmp_num_scores = $cmp_num_scores - $cmpHasGamma
		global cmpHasGamma 0
		mata _HasGamma = 0; _GammaInds = J($cmp_d, 1, &J(0,1,0))

		di as res _n "Fitting " plural($cmp_d>1, "constant") "-only model for LR test of overall model fit."
		qui InitSearch if `touse' `wgtexp', `svy' 1only  auxparams(`auxparams')
		local 1onlyinitconstraints `r(initconstraints)'
		mat `b' = r(b)
		qui Estimate `method_spec' if `touse' `wgtexp', cmpinit(`b') constraints(`_constraints' `1onlyinitconstraints') `autoconstrain' psampling(`psampling') resteps(`steps') ///
		                `svy' `mlopts' `technique' auxparams(`r(auxparams)') 1only `quietly' redraws(`redraws') paramsdisplay(`r(ParamsDisplay)') bounds(`bounds')
		if _rc==0 local lf0opt lf0(`e(rank)' `e(ll)')

		global cmpHasGamma `HasGamma'
		mata _HasGamma = $cmpHasGamma
		global cmp_num_scores = $cmp_num_scores + $cmpHasGamma
	}
	mata _GammaInds = __GammaInds // hidden from constants-only fit

	tempname LeftCens RightCens
	qui forvalues eq=1/$cmp_d {
		cap gen byte `LeftCens'  = _cmp_ind`eq'==$cmp_int & `touse' & ${cmp_y`eq'_L}>=.
		if _rc==0 {
			replace _cmp_ind`eq' = $cmp_left      if `LeftCens'  // Having gotten initial fits treating as intreg,
			cap gen byte `RightCens' = _cmp_ind`eq'==$cmp_int & `touse' & ${cmp_y`eq'  }>=.
			if _rc==0 {
				replace _cmp_ind`eq' = $cmp_right     if `RightCens' // helps speed & precision to treat left & right
				replace ${cmp_y`eq'}   = ${cmp_y`eq'_L} if `RightCens' // intreg cases as bounded on only one side
			}
		}

		global cmp_x`eq'  `cmp_x`eq''
		global cmp_xc`eq' `cmp_xc`eq''
	}

	if $parse_L==1 & $cmpSigXform {
		foreach param in `auxparams' {
			if substr("`param'", 2, 2) == "ln" {
				local diparmopt `diparmopt' diparm(`=substr("`param'", 2, .)', exp  label("`=substr("`param'", 4, .)'"))
			}
			else if substr("`param'", 2, 5) == "atanh" {
				local diparmopt `diparmopt' diparm(`=substr("`param'", 2, .)', tanh label("`=substr("`param'", 7, .)'"))
			}
		}
	}

	di as res _n "Fitting full model."
	cmp_full_model `method_spec' if `touse' `wgtexp', `vce' score(`score') `lf0opt' modopts(`modopts') mlopts(`mlopts') `technique'  paramsdisplay(`ParamsDisplay') xvarsall(`XVarsAll') ///
		constraints(`_constraints' `initconstraints' `constraints') init(`init') cmpinit(`cmpInitFull') `svy' `interactive' psampling(`psampling') ///
		`quietly' auxparams(`auxparams') cmdline(`"`cmdline'"') lnl(`lnl') resteps(`steps') redraws(`redraws') intpoints(`intpoints') bounds(`bounds') ///
		vsmp(`vsmp') meff(`meff') ghkanti(`ghkanti') ghkdraws(`ghkdraws') ghktype(`ghktype') diparmopt(`diparmopt')

	constraint drop `_constraints' `initconstraints' `1onlyinitconstraints'

	if "`e(cmd)'"=="cmp" Display, `diopts'
end

cap program drop Parse
program define Parse
	version 10.0
	cap version 11.0
	local tsfv = cond(c(version)==11, "fv", "ts")
	
	local _cons _cons
	global parse_d 0
	gettoken eq eqs: 0, match(parenflag)
	while `"`eq'"' != "" {
		global parse_d = $parse_d + 1

		tokenize `"`eq'"', parse(" :")
		if "`2'" == ":" {
			confirm name `1'
			global parse_eq$parse_d `1'
			macro shift 2
		}

		local eq `*'
		gettoken 0 eq: eq, parse("=|")
		if "`0'" != "|" { // includes an FE equation?
			`tsfv'unab myy: `0'
			global parse_y $parse_y `myy'
			global parse_y$parse_d `myy'
			
			gettoken 0 eq: eq, parse("=")
			if `"`0'"' != "=" cmp_error 198 `"Missing "=": (`0')"'
			
			gettoken 0 eq: eq, parse("|")
			if "`0'" == "|" {
				local 0
				local eq |`eq'
			}

			syntax [anything], [noCONStant OFFset(varname) EXPosure(varname) TRUNCpoints(string) iia]
			tokenize `anything'
			while `"`1'"' != "" {
				if substr(`"`1'"',strlen(`"`1'"'),1)=="#" global parse_yR$parse_d ${parse_yR$parse_d} `=substr(`"`1'"', 1, strlen(`"`1'"')-1)'
					else local varlist `varlist' `1'
				macro shift
			}
			if "`varlist'" != "" {
				`tsfv'unab varlist: `varlist'
				global parse_x$parse_d `varlist'
				global parse_x $parse_x `varlist'
				global parse_iia$parse_d `iia'
			}
			if "`constant'" != "" {
				global parse_xc$parse_d nocons
				if "${parse_x$parse_d}${parse_yR$parse_d}" == "" cmp_error 198 `"(`0')"'
			}
			if "`offset'"!="" | "`exposure'"!="" {
				if "`offset'" != "" & "`exposure'" != "" cmp_error 198 "Cannot specify both offset() and exposure()."
				global parse_xo$parse_d `offset'
				global parse_xe$parse_d `exposure'
			}
			if `"`truncpoints'"' != "" {
				tokenize `"`truncpoints'"'
				if `"`3'"' != "" | `"`2'"'=="" cmp_error 198 `"truncpoints(`"`truncpoints'"') invalid. Must have two arguments. Arguments with spaces must be quoted."'
				global parse_tr$parse_d `truncpoints'
			}
		}
		else local eq |`eq' // if no FE eq, stick the | back on the beginning
		
		gettoken 0 eq: eq, parse("|")
		while `"`0'"' != "" {
			gettoken 0 eq: eq, parse("|")
			if `"`0'"' != "|" cmp_error 198 `""|" not allowed in equation specification. Use "||"."'

			gettoken 0 eq: eq, parse("|")
			tokenize `"`0'"'
			local id `1'
			confirm name `id'
			if "`2'" != ":" cmp_error 198 `"Specify random effects starting with the group identifier variable and a ":"."'

			global parse_id: list global(parse_id) | id
			global parse_id$parse_d: list global(parse_id$parse_d) | id
			local L: list posof "`id'" in global(parse_id)
			macro shift 2
		
			local 0 `*'
			syntax [varlist(`tsfv' ts default=none)] [fw aw pw iw], [noCONStant COVariance(string)]

			if "`varlist'"!="" `tsfv'unab varlist: `varlist'
			local t: list varlist - global(parse_x$parse_d)

			if "${parse_wtype`L'}" == "" & `"${parse_wexp`L'}"' == ""  {
				global parse_wtype`L' `weight'
				global parse_wexp`L' `"`exp'"'
				if "`weight'"=="fweight" global parse_fweight 1
			}
			else if "${parse_wtype`L'}"!="`weight'" | `"${parse_wexp`L'}"!=`"`exp'"' cmp_error 198 "Weights more than once for the `id' level specified more than once."

			if "`varlist'"=="" & "`constant'"!="" di as res "Warning: No random effect or coefficients specified for the `id' level of the ${parse_y$parse_d} equation."

			if "`constant'" == "" global parse_re`L'_$parse_d _cons
			global parse_rc`L'_$parse_d `varlist'
			
			if inlist(`"`covariance'"', ".", "") global parse_cov`L'_$parse_d unstructured
			else {
				local 0, `covariance'
				syntax, [UNstructured EXchangeable INDependent]
				global parse_cov`L'_$parse_d `unstructured'`exchangeable'`independent'
				if inlist("${parse_cov`L'_$parse_d}", "unstructured", "exchangeable", "independent") == 0 {
					cmp_error 198 `"Each entry in the {cmdab:cov:ariance()} option must be "unstructured", "." (equivalent to "unstructured"), "exchangeable", or "independent"."'
				}
			}

			gettoken 0 eq: eq, parse("|")
		}
		global parse_id$parse_d ${parse_id$parse_d} _n

		gettoken eq eqs: eqs, match(parenflag)
	}

	global parse_L: word count $parse_id

	if $parse_L > 1 { // draw together partial orderings implied by random effect sequences in each equation
		mata _X = I($parse_L)
		forvalues i=1/$parse_d {
			tokenize ${parse_id`i'}
			forvalues j=1/`=`:word count ${parse_id`i'}'-2' {
				mata _X[`:list posof "``j''" in global(parse_id)',`:list posof "``=`j'+1''" in global(parse_id)'] = 1
			}
		}
		mata _Y = _X; for (i=$parse_L-1; i; i--) _Y = _Y * _X
		mata _p = order(rowsum(_Y:!=0), -1) // permutation to make Y upper triangular
		mata st_local("t", strofreal(all(vech(_Y'[_p,_p]))))
		if `t' mata st_global("parse_id", invtokens(tokens("$parse_id")[_p]))
		else {
			di as err "Cannot determine hierarchical order of levels."
			di as err `"You can add a dummy equation like ""' as res "( || id1: || id2: || id3:)" as err `"" to specify the full ordering."'
			di as err "Don't include this equation in the " as res "indicators()" as err " option."
			cmp_error 110
		}
	}

	local i 0
	forvalues j=1/$parse_d { // expunge dummy eqs
		if "${parse_y`j'}" != "" {
			if `++i' != `j' {
				foreach macro in eq y yR x xo xe xc tr id {
					global parse_`macro'`i' ${parse_`macro'`j'}
					global parse_`macro'`j'
				}
				forvalues l=1/$parse_L {
					foreach macro in rc re {
						global parse_`macro'`l'_`i' ${parse_`macro'`l'_`j'}
						global parse_`macro'`l'_`j'
					}
				}
			}

			if "${parse_eq`i'}" == "" {
				global parse_eq`i' eq`i'
			}
			local eqnames `eqnames' ${parse_eq$`i'}
		}
	}
	global parse_d `i'
	forvalues j=1/$parse_d { // will cause to generate and extract /atanhrho params to work correctly for bottom level
		global parse_re$parse_L_`j' _cons
	}
	
	local t: list dups eqnames
	if "`t'" != "" cmp_error 110 "Multiply defined equations: `t'"
	
	global parse_id $parse_id _n
	global parse_L: word count $parse_id
end

* These lines are in a subroutine to work around Stata parsing bug with "if...quietly {"
cap program drop DoInitSearch
program define DoInitSearch, rclass
	version 10.0
	cap version 11.0
	di as res "Note: For programming reasons, these initial estimates may deviate from your specification."
	di as res "      For exact fits of each equation alone, run cmp separately on each."
	`*' // run InitSearch
	tempname b
	mat `b' = r(b)
	return matrix b = `b'
	return local auxparams `r(auxparams)'
	return local initconstraints `r(initconstraints)'
	return local ParamsDisplay `r(ParamsDisplay)'
	return local XVarsAll `r(XVarsAll)'
end

* perform full estimate. Program cmp is not eclass, so it can be called for non-estimating purposes without obliterating current estimates
* cmp_full_model is eclass, so it performs and saves full estimate
cap program drop cmp_full_model
program define cmp_full_model, eclass
	version 10.0
	cap version 11.0
	syntax anything if/ [pw fw aw iw], [auxparams(string) vsmp(string) meff(string) paramsdisplay(string) xvarsall(string) ///
					ghkanti(string) ghkdraws(string) ghktype(string) diparmopt(string) cmdline(string) ///
					redraws(string) resteps(string) retype(string) reanti(string) intpoints(string) * ]

	Estimate `anything' if `if' [`weight'`exp'], auxparams(`auxparams') paramsdisplay(`paramsdisplay') resteps(`resteps') redraws(`redraws') `options'

	if _rc==0 {
		mata _interactive = 1 // in case cmp_lf or cmp_lf1 reentered post-estimation

		if "`meff'" != "" _svy_mkmeff `vsmp'

		if $cmpHasGamma { // prepare to build reduced-form b and V
			tempname b beq Beta Gamma
			mat `b' = e(b)
			mat `Gamma' = J($cmp_d, $cmp_d, 0)
			if "`xvarsall'" != "" mat `Beta'  = J($cmp_d, `:word count `xvarsall'', 0)
			mat cmpBetaInd  = .,.
			local cols 0
			forvalues eq=1/$cmp_d {
				mat `beq' = `b'[1,"`:word `eq' of $cmp_eq':"] // `b'[1,"#`eq'"] doesn't work in Stata 10
				local colnames: colnames `beq'
				local cols = `cols' + colsof(`beq')
				forvalues i=1/`=colsof(`beq')' {
					local var: word `i' of `colnames'
					local j: list posof "`var'" in xvarsall
					mat `Beta'[`eq',`j'] = `beq'[1,`i']
					mat cmpBetaInd  = cmpBetaInd  \ `eq',`j'
				}
			}
			forvalues i=1/`=rowsof(cmpGammaInd)-1' {
				mat `Gamma'[cmpGammaInd[`i',1], cmpGammaInd[`i',2]] = `b'[1,`cols'+`i']
			}
			mat cmpGammaInd = .,. \ cmpGammaInd[1..`=rowsof(cmpGammaInd)-1', 1...]
		}

		mata cmpSaveSomeResults() // Get final Sig; if weights, get weighted sample size; for Gamma models build e(br), e(Vr)

		if $cmpHasGamma { // eliminate unnecessary "#"'s in e(b) colnames, unnecessary for predict that is, which wrongly imply that variable is unobserved
			mat colnames `b' = `paramsdisplay'
			local paramsdisplay: colnames `b'
			forvalues eq=1/$cmp_d {
				local eqname `:word `eq' of $cmp_eq'
				local eqnamep `eqname'#
				if "`:list eqnamep & paramsdisplay'" != "" {
					qui count if _cmp_ind`eq'>1 & e(sample)
					if r(N)==0 local paramsdisplay: subinstr local paramsdisplay "`eqname'#" "`eqname'", word all
				}
			}
			mat colnames `b' = `paramsdisplay'
			ereturn local params `:colfullnames `b''
		}
		
		if "`1only'"=="" & "`e(chi2type)'"=="Wald" {
			local t
			forvalues eq=1/$cmp_d {
				cap test ([${cmp_eq`eq'}]) // crashes if cons only
				if _rc==0 local t `t' ([${cmp_eq`eq'}])
			}
			if "`t'" != "" {
				qui test `t'
				ereturn scalar df_m = r(df)
				ereturn scalar p = r(p)
				ereturn scalar chi2 = r(chi2)
			}
		}

		ereturn local wexp `"$parse_wexpL"'
		forvalues eq=1/$cmp_d {
			qui count if e(sample) & _cmp_ind`eq'
			ereturn scalar N`eq' = r(N)
		}

		tempname t
		mat `t' = J($cmp_d, $cmp_max_cuts+1, .)
		mat rownames `t' = $cmp_eq
		forvalues i=1/$cmp_d {
			if cmp_num_cuts[`i',1] {
				forvalues j=1/`=cmp_num_cuts[`i',1]+1' {
					mat `t'[`i',`j'] = `:label (_cmp_y`i') `j''
				}
			}
		}
		ereturn mat cat = `t'
		mat `t' = cmp_num_cuts
		mat rownames `t' = $cmp_eq
		ereturn mat num_cuts = `t'

		if "``intpoints'" != "" {
			ereturn local n_quad `intpoints'
			ereturn local quad_method = cond($cmp_intmethod, "mvaghermite", "ghermite")
		}
	
		ereturn scalar num_scores = $cmp_num_scores
		ereturn scalar sigxform = $cmpSigXform
		ereturn scalar L = $parse_L
		ereturn local ivars $parse_id
		ereturn mat NumEff = cmp_NumEff
		forvalues l=1/$parse_L {
			ereturn mat fixed_rhos`l' = cmp_fixed_rhos`l'
			ereturn mat fixed_sigs`l' = cmp_fixed_sigs`l'
			cap ereturn mat sig_score_inds`l' = cmpSigScoreInds`l'
			ereturn local covariance `e(covariance)' ${cmp_cov`l'}
			forvalues eq=1/$cmp_d {
				ereturn local covariance`eq' `e(covariance`eq')' ${cmp_cov`l'_`eq'}
				if `l'<$parse_L ereturn local EffNames`l'_`eq' = cond(`l'==$parse_L, "_cons", "${cmp_rc`l'_`eq'} ${cmp_re`l'_`eq'}")
			}
		}

		foreach macro in diparmopt ghkanti ghkdraws ghktype retype reanti intpoints {
			ereturn local `macro' ``macro''
		}
		if `resteps' > 1 ereturn local resteps `resteps'
		ereturn local depvar $parse_y
		ereturn local indicators = `"`indicators'"'
		ereturn local eqnames $cmp_eq `:subinstr local auxparams "/" "", all'
		ereturn local predict cmp_p
		ereturn local title Mixed-process `=cond($parse_L>1, "multilevel ", "")'regression
		ereturn local cmdline cmp `cmdline'
		ereturn local cmd cmp
	}
	cmp_clear
	if _rc==1 error 1
end

cap program drop NoEstimate
program NoEstimate, eclass
	version 10.0
	ereturn post `0'
	ereturn local title Mixed-process regression--initial fits only
	ereturn local cmdline cmp `cmdline'
	ereturn local cmd cmp
end

cap program drop Estimate
program Estimate, eclass
	version 10.0
	cap version 11.0
	syntax anything(name=method_spec) if/ [fw aw pw iw], [auxparams(string) psampling(string) svy score(string) interactive autoconstrain paramsdisplay(string) bounds(string) ///
		modopts(string) mlopts(string) init(string) cmpinit(string) constraints(string) technique(string) 1only quietly lnl(string) resteps(string) redraws(string) *]

	if "`weight'" != "" local awgtexp [aw`exp']
	tempname _init
	if "`init'" == "" local _init `cmpinit'
		else {
			mat `_init' = `init'
			cap mat colnames `_init' = `:colfullnames `cmpinit''
			if _rc cmp_error 503 "init() matrix should 1 x `:word count `:colfullnames `cmpinit'''."
		}

	local mlversion = cond(c(stata_version)>=11 & substr("`method_spec'",1,2)!="d2", "11", "`c(version)'")

	if $cmpSigXform {
		local ln ln
		local atanh atanh
	}

/*	if "`constraints'" != "" {
		tempname b V Cns CnsCoef CnsCoefCols a C
		mat `b' = `cmpinit'
		mat `V' = `b'' * `b'
		ereturn post `b' `V'
		makecns `constraints'
		if "`e(Cns)'" == "" local Cns // if all constraints bad, no e(Cns) made
		else {
			mat `Cns' = e(Cns)
			if $parse_L>1 & "`autoconstrain'"=="" {
				mata _Cns = st_matrix("e(Cns)")
				// probit of y on nothing makes $cmpNumCoefs=0 and next statement crashes
				mata _Cns = select(_Cns, rowsum(abs(_Cns[,1.._NumCoefs]):==1):==2) // constraints that equate two coefficients (up to sign)
				mata st_matrix("`CnsCoefCols'", rowminmax(editvalue(abs(_Cns) :* J(rows(_Cns), 1, 1..cols(_Cns)), 0, .))) // cols affected by each constraint (2 each)
				local colnames: colfullnames `cmpinit'
				forvalues l=1/`=$parse_L-1' {
					mata __Cns = _Cns
					local NewCnsCt 0
					forvalues c1=1/$cmpNumCoefs { // Use Guassian elimination to minimize constraints refering to coefs lacking /lnsigs in this level
						local eq1         : list posof "`:word `c1' of `:coleq    `cmpinit'''" in global(cmp_eq)
						local coef1       : list posof "`:word `c1' of `:colnames `cmpinit'''" in global(cmp_x`eq1')
						local lnsigcol`c1': list posof `"`ln'sig`=cond(`coef1',"_`coef1'","")'_`l'_`eq1':_cons"' in colnames
						if `lnsigcol`c1''==0 mata __Cns = eliminate(__Cns, `c1')
						forvalues c2=1/$cmpNumCoefs { // Use Guassian elimination to minimize constraints refering to coefs lacking /lnsigs in this level
							local eq2     : list posof "`:word `c2' of `:coleq    `cmpinit'''" in global(cmp_eq)
							local coef2   : list posof "`:word `c2' of `:colnames `cmpinit'''" in global(cmp_x`eq2')
							local eqlabel = cond(`coef1'+`coef2' | `eq1'==`eq2', `"_`=cond(`coef1', "`coef1'", "`=1+`:word count ${cmp_x`eq1'}''")'_`=cond(`coef2', "`coef2'", "`=1+`:word count ${cmp_x`eq2'}''")'"', "")
							local atanhrhocol`c1'_`c2': list posof `"`atanh'rho`eqlabel'_`l'_`eq1'`=cond(`eq1'==`eq2',"","`eq2'")':_cons"' in colnames
						}
					}
					mata st_matrix("`CnsCoef'", __Cns)
					cap confirm matrix `CnsCoef'
					if _rc==0 {
						forvalues r1=1/`=rowsof(`CnsCoef')' { // auto-constrain corresponding lnsigs
							mat `Cns' = `Cns' \ J(1, colsof(`Cns'), 0)
							local ++NewCnsCt
							forvalues ci1=1/2 {
								local c1 = `CnsCoefCols'[`r1',`ci1']
								cap mat `Cns'[`=rowsof(`Cns')', `lnsigcol`c1'] = `CnsCoef'[`r1',`c1']
								if _rc {
									mat `Cns' = `Cns'[1..`=rowsof(`Cns')-1', 1...]
									local --NewCnsCt
									continue, break
								}
							}
							forvalues r2=1/`=rowsof(`CnsCoef')' { // auto-constrain atanhrhos corresponding to pairs of constraints
								mat `Cns' = `Cns' \ J(1, colsof(`Cns'), 0)
								local ++NewCnsCt
								forvalues ci1=1/2 {
									local c1 = `CnsCoefCols'[`r1',`ci1']
									forvalues ci2=1/2 {
										local c2 = `CnsCoefCols'[`r1',`ci2']
										cap mat `Cns'[`=rowsof(`Cns')', ``atanhrhocol`c1'_`c2''] = (-1)^(`c1'+`c2')
										if _rc {
											mat `Cns' = `Cns'[1..`=rowsof(`Cns')-1', 1...]
											local --NewCnsCt
											continue, break
										}
										if _rc continue, break
									}
								}
							}
						}
					}
				}
			}
		}
	}*/

	if $cmpHasGamma {
		local _paramsdisplay `paramsdisplay'
		if "`constraints'" != "" { // as necessary, translate constraints expressed in terms of displayed names into terms of /gamma_X_Y
			tempname b V T a C Cns
			foreach c of numlist `constraints' { // make temporary copy of constraint set
				constraint free
				constraint `r(free)' `:constraint `c''
				local _constraints `_constraints' `r(free)'
			}
			local i 0
			forvalues eq=1/$cmp_d { // makeCns doesn't like names suffixed with #; rename them temporarily
				tempname v`++i'
				foreach c of numlist `_constraints' {
					local thisConstraint: constraint `c'
					constraint `c' `:subinstr local thisConstraint "${cmp_eq`eq'}#" "`v`i''", all'
				}
				local _paramsdisplay: subinstr local _paramsdisplay "${cmp_eq`eq'}#" "`v`i''", all
			}
			mat `b' = `_init'
			mat colnames `b' = `_paramsdisplay'
			mat `V' = `b'' * `b'
			ereturn post `b' `V'
			makecns `_constraints'
			matcproc `T' `a' `C'
			mat `b' = e(b)
			mat `V' = e(V)
			eret post `b' `V' `C'
			mat `b' = e(b)
			mat colnames `b' = `:colfullnames `_init''
			eret repost b = `b', rename
			mat `Cns' = e(Cns)
			constraint drop `_constraints'
			local constraints `Cns'
		}
	}

	forvalues eq=1/$cmp_d { 
		local model `model' (${cmp_eq`eq'}: ${cmp_y`eq'} = 
		if "`1only'"=="" local model `model' ${cmp_x`eq'}
		local model `model', ${cmp_xc`eq'} offset(${cmp_xo`eq'}) exposure(${cmp_xe`eq'}))
	}
	if "`1only'"=="" local model `model' $cmp_gammaparams
	local model `method_spec' `model' `auxparams'
	local modeldisplay `model'

	if "`interactive'" == "" & "`constraints'" == "" {
		local model
		forvalues eq=1/$cmp_d {
			if "`1only'"=="" local xvars `xvars' ${cmp_x`eq'}
			if "${cmp_xc`eq'}"=="" local xvars `xvars' _cons
			local tsfv = cond(c(version)==11, "fv", "ts")
			foreach macro in y x xo xe {
				if "${cmp_`macro'`eq'}" != "" & ("`macro'"!="x" | "`1only'"=="") {
					`tsfv'revar ${cmp_`macro'`eq'}
					if c(stata_version)>=11 { // stick "o." back on fvrevar'd omitted variables
						tokenize `r(varlist)'
						fvexpand ${cmp_`macro'`eq'} if _cmp_ind`eq'
						local `macro'_revar
						forvalues i=1/`:word count `r(varlist)'' {
							local `macro'_revar ``macro'_revar' `=cond(strpos("`:word `i' of `r(varlist)''", "o."), "o.", "")'``i''  
						}
					}
					else local `macro'_revar `r(varlist)'
				}
				else local `macro'_revar
			}
			forvalues l=1/$parse_L {
				if "${cmp_rc`l'_`eq'}" != "" {
					`tsfv'revar ${cmp_rc`l'_`eq'}
					global cmp_rc`l'_`eq' `r(varlist)'
				}
			}
			local model `model' (${cmp_eq`eq'}: `y_revar' = `x_revar', ${cmp_xc`eq'} offset(`xo_revar') exposure(`xe_revar'))
*			local x_revar_all `x_revar_all' `x_revar'
		}
		if "`1only'"=="" local model `model' $cmp_gammaparams
		local model `method_spec' `model' `auxparams'
/*		if "`Cns'" != "" {
			mata _colnames = st_matrixcolstripe("`Cns'")
			mata _t = tokens("`x_revar_all'")
			mata _colnames[|1,2 \ length(_t),2|] = _t'
			mata st_matrixcolstripe("`Cns'", _colnames)
		}
*/
	}

	tempname b sample
	mata _intmethod = 0
	forvalues quadfinal=0/`=!!$cmp_intmethod' {
		if "`psampling'" == "" {
			local psampling_cutoff 1
			local psampling_rate 2
			local u 1
		}
		else {
			count if `if'
			local N = r(N)
			tokenize `psampling'
			local psampling_cutoff = cond(`1'>=1, `1'/`N', `1')
			local psampling_rate = cond(0`2', 0`2', 2)
			tempvar u
			gen `u' = uniform() if `if'
		}

		while `psampling_cutoff' < `psampling_rate' {
			if "`psampling'" != "" {
				if `psampling_cutoff' < 1 di as res _n "Fitting on approximately " %1.0f `psampling_cutoff'*`N' " observations (approximately " %1.0f `psampling_cutoff'*100 "% of the sample)."
				else di as res _n "Fitting on full sample."
			}

			if `resteps'>1 mata __NumREDraws = J(`=$parse_L-1', 1, 1) :/ (_DrawMultipliers = strtoreal(tokens("`redraws'"))' :^ (1/(`resteps'-1)))

			forvalues restep = 1/`resteps' {
				if `restep' < `resteps' mata __NumREDraws = __NumREDraws:*_DrawMultipliers
													 else mata __NumREDraws = strtoreal(tokens("`redraws'"))'

				if "`_init'" != "" local initopt init(`_init', copy)

				mata _NumREDraws = 1 \ ceil(__NumREDraws) * _REAnti

				local final = `psampling_cutoff'>=1 & `restep'==`resteps' & `quadfinal'==!!$cmp_intmethod
				if `final' {
					local this_mlopts `mlopts'
					local this_technique `technique'
				}
				else {
					local this_mlopts nonrtolerance tolerance(0.1)
					local this_technique nr
				}

				mata _first_call = 1
				if "`interactive'" == "" {
					preserve
					qui keep if (`if') & (`psampling_cutoff'>=1 | `u'<=.001+`psampling_cutoff')
					local mlcmd `quietly' version `mlversion': ml model `model' `=cond(`final',"[`weight'`exp'], `options' score(`score')", "`awgtexp',")' max ///
						`svy' constraints(`constraints' /*`Cns'*/) nocnsnotes nopreserve missing collinear `this_mlopts' `modopts' bounds(`bounds') technique(`this_technique')
					capture noisily `mlcmd' `initopt' search(off)
					if _rc==1400 {
						di as res "Restarting search with parameters all 0."
						tempname zeroes
						mat `zeroes' = J(1, `=colsof(`_init')', 0)
						mata _first_call=1
						capture noisily `mlcmd' init(`zeroes', copy) search(off)
					}
					restore

					if _rc==1 {
						local rc = _rc
						cmp_clear
						error `rc'
					}
					if _rc continue

					mat `b' = e(b)

					if "`constraints'" == "" | $cmpHasGamma mat colnames `b' = `:colnames `cmpinit''

					gen byte `sample' = `if'
					ereturn repost b = `b', rename esample(`sample')
					cap _ms_op_info e(b)
					if _rc==0 & r(fvops) {
						ereturn repost, buildfvinfo ADDCONS
						ereturn local marginsprop `e(marginsprob)' addcons
					}
					cap local _a // reset _rc to 0
				}
				else {
					local mlmodelcmd version `mlversion': ml model `model' `=cond(`final',"[`weight'`exp'] if `if',`options'","`awgtexp' if (`if') & `u'<=.001+`psampling_cutoff', ")' ///
						`svy' constraints(`Cns') nocnsnotes `modopts' collinear technique(`this_technique') nopreserve missing
					local mlmaxcmd `quietly' version `mlversion': ml max, search(off) noclear nooutput `this_mlopts' `=cond(`psampling_cutoff'>=1, "score(`score')", "")' bounds(`bounds')
					`mlmodelcmd' `initopt'
					`mlmaxcmd'
					if _rc==1400 {
						di as res "Restarting search with initial parameters all 0."
						tempname zeroes
						mat `zeroes' = J(1, `=colsof(`_init')', 0)
						mata _first_call=1
						`mlmodelcmd' init(`zeroes', copy)
						capture noisiliy `mlmaxcmd'
					}
					else if _rc==1 {
						cmp_clear
						error 1
					}
					if _rc continue
				}
				if `restep' < `resteps' {
					if "`quietly'"=="" noi version `mlversion': ml di
					mat `_init' = e(b)
				}
			} //resteps loop

			local psampling_cutoff = `psampling_cutoff' * `psampling_rate'
			if `psampling_cutoff' < `psampling_rate' {
				noi version `mlversion': ml di
				mat `_init' = e(b)
			}
		} // psampling loop

		if $cmp_intmethod {
			mat `_init' = e(b)
			mata _intmethod = $cmp_intmethod
		}
	} // two-step adaptive quadrature loop
	
	ereturn scalar k_gamma = $cmpHasGamma
	ereturn scalar k_aux = `:word count `auxparams'' + e(k_gamma)
	ereturn scalar k_eq = $cmp_d + e(k_aux)
	
	if _rc {
		local rc = _rc
		cmp_clear
		error `rc'
	}

	if "`lnl'" != "" {
		quietly version `mlversion': ml model `model' [`weight'`exp'] if `if', `options' `svy' nopreserve missing collinear `modopts'
		tempname b
		mat `b' = e(b)
		cmp_lf1 0 `b' `lnl'
		ml clear
	}

	if "`interactive'" != "" ereturn repost, esample(`if')
	
	ereturn local model `modeldisplay'
end

cap program drop LabelMprobitEq
program LabelMprobitEq
	version 10.0
	// try to name the eq after the outcome's label
	global cmp_eq`1' : label cmp_y`2'_label `4'
	cap confirm names ${cmp_eq`1'}
	if _rc | `:word count ${cmp_eq`1'}' > 1 global cmp_eq`1' _outcome_`3'_`4'
end

// Given current ml model, estimate starting points equation by equation
// Also, return reduced lists of RHS vars reflecting rmcoll and perfect-prediction eliminations in probit case
// as well as constraints on rho's needed for equations with non-overlapping samples
cap program drop InitSearch
program InitSearch, rclass
	version 10.0
	cap version 11.0
	syntax [aw fw iw pw] if/, [auxparams(string) adjustprobitsample nodrop svy 1only quietly]
	tempname sig beta gamma betavec gammavec auxparamvec _auxparamvec sigvec atanhrho V mat_cons omit
	tempvar y id choice
	local _cons _cons
	mat `mat_cons' = 0
	mat `auxparamvec' = 0
	mat colnames `mat_cons' = "_cons"
	if "`svy'"!="" local svy svy:
	if "`weight'" != "" & "`svy'"=="" {
		local iwgtexp [iweight `exp']
		local awgtexp [aweight `exp']
	}

	if $cmpSigXform {
		local ln ln
		local atanh atanh
	}

	forvalues eq=1/$cmp_d {
		local xvars
		tempvar e`eq' ebar`eq'

		qui {
			count if `if' & _cmp_ind`eq'==$cmp_oprobit
			if r(N) local regtype $cmp_oprobit
			else {
				count if `if' & (_cmp_ind`eq' > $cmp_mprobit_ind_base & _cmp_ind`eq' < $cmp_roprobit_ind_base) | inlist(_cmp_ind`eq', $cmp_probit, $cmp_probity1, $cmp_mprobit)
				if r(N) local regtype $cmp_probit
				else {
					count if `if' & (_cmp_ind`eq' == $cmp_int)
					if r(N) local regtype $cmp_int
					else {
						count if `if' & (_cmp_ind`eq' == $cmp_roprobit)
						if r(N) local regtype $cmp_roprobit
						else {
							count if `if' & (_cmp_ind`eq' == $cmp_left | _cmp_ind`eq' == $cmp_right)
							local regtype = cond(r(N), $cmp_left, $cmp_cont)
						}
					}
				}
			}
		}

		if "`1only'"=="" {
			if c(version) >= 11 {
				fvexpand ${cmp_x`eq'} if `if' & _cmp_ind`eq'
				global cmp_x`eq' `r(varlist)'
			}

			if "`drop'" == "" {
				`=cond(`regtype'==1, "_rmdcoll ${cmp_y`eq'}", "_rmcoll")' ${cmp_x`eq'} if `if' & _cmp_ind`eq', ${cmp_xc`eq'} `=cond(c(version)>=11, "expand", "")'
				foreach var in `r(varlist)' {
					if strpos("`var'", "o.") == 0 local xvars `xvars' `var'
				}
			}
			else local xvars ${cmp_x`eq'}
		}

		forvalues tries=0/1 { // Try at most two times to get a regression without collinear terms, perfect predictors, missing standard errors, etc.
			local keep
			cap drop `e`eq''
			cap mat drop `_auxparamvec'
			if `regtype'==$cmp_oprobit {
				`quietly' `svy' oprobit ${cmp_y`eq'} `xvars' `iwgtexp' if `if' & _cmp_ind`eq', offset(${cmp_xo`eq'})
				mat `beta' = e(b)
				mat `V' = e(V)
				scalar `sig' = 0
				mat `_auxparamvec' = nullmat(`_auxparamvec'), `beta'[1, `=colsof(`beta')-e(k_aux)+1'...]
				if e(k_eq) == e(k_cat) {
					mat `beta' = `beta'[1,"${cmp_y`eq'}:"]
					mat `V' = `V'["${cmp_y`eq'}:","${cmp_y`eq'}:"]
					global cmp_xc`eq' nocons
				}
				else {
					mat `beta' = `mat_cons' // put a 0 on end for "constant"
					constraint free
					local initconstraints `initconstraints' `r(free)'
					constraint `r(free)' [${cmp_eq`eq'}]_cons
					global cmp_xc`eq'
				}
				qui if $cmp_d > 1 | $parse_L > 1 {
					_predict `e`eq'' if e(sample)
					recode `e`eq'' (. = 0) if e(sample) // can be all missing if there are no regressors
					replace `e`eq'' = ${cmp_y`eq'} - `e`eq''
				}
			}
			else if `regtype'==$cmp_probit {
				cap confirm variable _mp_cmp_y`eq' // check for cmp-made dummies for non-as mprobit
				`quietly' `svy' probit `=cond(_rc, "${cmp_y`eq'}", "_mp_cmp_y`eq'")' `xvars' `iwgtexp' ///
					if `if' & (inlist(_cmp_ind`eq',$cmp_probit,$cmp_probity1,$cmp_mprobit) | (_cmp_ind`eq' > $cmp_mprobit_ind_base & _cmp_ind`eq' < $cmp_roprobit_ind_base)), ${cmp_xc`eq'} offset(${cmp_xo`eq'})
				if "`adjustprobitsample'" != "" qui replace _cmp_ind`eq' = 0 if e(sample)==0
				mat `beta' = e(b)
				mat `V' = e(V)
				scalar `sig' = 0
				qui if $cmp_d > 1 | $parse_L > 1 {
					predict `e`eq'' if e(sample)
					replace `e`eq'' = (`e(depvar)'!=0) - `e`eq''
				}
				tempname rules
				mat `rules' = e(rules)

				local imperfect
				foreach var in `:colnames `beta'' {
					if strpos("`var'", "o.") == 0 local imperfect `imperfect' `var'
				}

				foreach var in `:list xvars - imperfect' {
					cap mat `rules2' = `rules'["`var'",1] // error can occur if using FV's and nodrop. Some entries of xvars have "o." but are dropped from regression
					if _rc==0 {
						if `rules2'[1,1]==1 {
							di as res _n "Warning: `var' perfectly predicts success or failure in ${cmp_y`eq'}."
							if "`drop'" == "" di as res "It will be dropped from the full model."
							di as res "Perfectly predicted observations will be dropped from the estimation sample for this equation."
						}
						else if "`drop'" != "" & c(version)<11 {  // keep collinear regressors?
							local keep `keep' `var'
						}
					}
				}
			}
			else if `regtype'==$cmp_left {
				cap noisily `svy' tobit ${cmp_y`eq'} `xvars' `awgtexp' if `if' & inlist(_cmp_ind`eq', 1, 2, 3), ${cmp_xc`eq'} ll ul
				if _rc & _rc != 430 { // crash on error other than failure to converge
					error _rc
					cmp_error _rc
				}
				mat `beta' = e(b)
				scalar `sig' = ln([sigma]_cons)
				mat `beta' = `beta'[1, "model:"]
				mat `V' = e(V)
				mat `V' = `V'["model:", "model:"]
				qui if $cmp_d > 1 | $parse_L > 1 {
					predict `e`eq'' if e(sample) & (${cmp_y`eq'}>`e(llopt)' | `e(llopt)'==.) & ${cmp_y`eq'}<`e(ulopt)'
					replace `e`eq'' = ${cmp_y`eq'} - `e`eq''
				}
			}
			else if `regtype'==$cmp_int {
				cap `svy' intreg ${cmp_y`eq'_L} ${cmp_y`eq'} `xvars' `iwgtexp' if `if' & inlist(_cmp_ind`eq', 1, $cmp_int), ${cmp_xc`eq'} offset(${cmp_xo`eq'})
				if _rc & _rc != 430 { // crash on error other than failure to converge
					error _rc
					cmp_error _rc
				}
				if "`quietly'"=="" intreg
				mat `beta' = e(b)
				scalar `sig' = [lnsigma]_cons
				mat `beta' = `beta'[1, "model:"]
				mat `V' = e(V)
				mat `V' = `V'["model:", "model:"]
				qui if $cmp_d > 1 | $parse_L > 1 {
					predict `e`eq'' if e(sample)
					replace `e`eq'' = cond(${cmp_y`eq'}<., cond(${cmp_y`eq'_L}<., (${cmp_y`eq'} - ${cmp_y`eq'_L})/2, ${cmp_y`eq'}), ${cmp_y`eq'_L}) - `e`eq''
				}
			}
			else { // uncensored or roprobit var
				`quietly' `svy' regress ${cmp_y`eq'} `xvars' `iwgtexp' if `if' & (_cmp_ind`eq'==1 |  _cmp_ind`eq'>=$cmp_roprobit_ind_base), ${cmp_xc`eq'}
				mat `beta' = e(b)
				if $cmp_reverse & `regtype'==$cmp_roprobit mat `beta' = -`beta'
				if "`svy'"=="" scalar `sig' = ln(e(rmse))
					else qui {
						tempname e
						predict `e' if e(sample), resid
						svyset
						sum `e' [iw `r(wexp)'] if e(sample)
						scalar `sig' = ln(r(sd))
					}
				mat `V' = e(V)
				if $cmp_d > 1 | $parse_L > 1 qui predict `e`eq'' if e(sample), resid
			}
			local dropped
			local k = colsof(`beta')
			if (`k') {
				local xvars : colnames `beta'
				local xvars : list xvars - `_cons'
				if diag0cnt(`V') & diag0cnt(`V') < rowsof(`V') { // unless all the coefs had se=0, drop those that did from this equation
					if "`drop'" == "" {
						if c(stata_version)>=11 {
							_ms_omit_info `V'
							mat `omit' = r(omit)
						}
						else mat `omit' = J(1, `k', 0)
						
						forvalues j=1/`=`k' - ("${cmp_xc`eq'}"=="")' {
							if `V'[`j',`j'] == 0 & `omit'[1,`j']==0 {
								if `"`dropped'"' == "" {
									di as res _n "Warning: Covariance matrix for single-equation estimate of ${cmp_y`eq'} equation is not of full rank."
									di as res "Parameters with singular variance will be excluded from full model."
									if `tries'==0 di as res "Re-running single-equation estimate without them."
								}
								local dropped `dropped' `:word `j' of `xvars''
								if c(stata_version)>=11 & `tries'==0 mat `beta'[1,`j'] = 0
							}
						}
						if c(stata_version)>=11 {
							_ms_findomitted `beta' `V' // In FV versions of Stata, prefix with "o." rather than dropping
							local xvars : colnames `beta'
							local xvars : list xvars - `_cons'
						}
						else local xvars : list xvars - dropped

						if "${cmp_xc`eq'}"=="" & `V'[`k',`k']==0 {
							local dropped `dropped' _cons
							global cmp_xc`eq' noconstant
						}
					}
					else {
						di as res "Parameters with singular variance will be retained in full model."
						local xvars " `:subinstr local xvars "o." ".", all'"
						local xvars: subinstr local xvars " ." " ", all
						local colnames: colnames `beta'
						local colnames " `:subinstr local colnames "o." ".", all'"
						mat colnames `beta' = `:subinstr local colnames " ." " ", all'
					}
				}
			}
			else local xvars
			if "`dropped'" == "" continue, break
		}
		mat `auxparamvec' = `auxparamvec', nullmat(`_auxparamvec')

		if "`keep'" != "" & c(version) < 11 {   // add back collinear regressors if nodrop specified
			if "${cmp_xc`eq'}"=="" mat `beta' = `=cond(`k'>1,"`beta'[1, 1..`=`k'-1'],", "")'  J(1, `:word count `keep'', 0), `beta'[1,`k']
			else                   mat `beta' = `beta', J(1, `:word count `keep'', 0)
		}

		if `k' mat coleq `beta' = ${cmp_eq`eq'}

		local t = cond("${cmp_yR`eq'}"=="" | "`1only'"!="", "", "${cmp_yR`eq'}") + cond(`k', " ", "")
		local t " `:subinstr local t " " "# ", all'`:colnames `beta''"
		local ParamsDisplay `ParamsDisplay' `:subinstr local t " " " ${cmp_eq`eq'}:", all'
		if `:word count ${cmp_yR`eq'}' & "`1only'"=="" {
			mat `gamma' = J(1, `:word count ${cmp_yR`eq'}', 0)
			mat `gammavec' = nullmat(`gammavec'), `gamma'
		}

		if cmp_fixed_sigs$parse_L[1,`eq']==. mat `sigvec' = nullmat(`sigvec'), cond($cmpSigXform, `sig', exp(`sig'))

		if e(converged) == 0 {
			di as res _n "Single-equation estimate for ${cmp_y`eq'} equation did not converge."
			di "This may indicate convergence problems for the full model too."
		}
		CheckCondition `xvars'

		if e(df_m) | `regtype'!=$cmp_oprobit {
			if "${cmp_xc`eq'}"=="" & `V'[`=colsof(`V')', `=colsof(`V')']==0 & e(converged) & diag0cnt(`V') < rowsof(`V') global cmp_xc`eq' noconstant
		}
		global cmp_x`eq' `xvars' `keep'

		if $cmpHasGamma {
			local t: colnames `beta'
			local XVarsAll: list XVarsAll | t
		}

		if colsof(`beta') {
			cap mat `betavec' = nullmat(`betavec'), `beta'
			if _rc {
				if _rc==198 & c(stata_version)>11 {
					di as err _n "Error constructing parameter vector. Possible cause:"
					di as err `"The base for an indicator ("i.") variable has a different minimum value"'
					di as err "in the samples for different equations, so Stata's default choice for base/omitted dummy "
					di as err "differs by equation. Fix: manually {help fvvarlist##bases:set a uniform base}."
					di as err `"E.g. to make the omitted case 2, prefix throughout with "ib2." instead of "i."."' _n
				}
				cmp_error _rc
			}
		}
	}

	if $cmpHasGamma {
		mat colnames `gammavec' = _cons
		mat coleq `gammavec' = `:subinstr global cmp_gammaparams "/" "", all'
		mat `betavec' = `betavec', `gammavec'
	}
	
	if "`adjustprobitsample'" != "" & $cmp_num_mprobit_groups { // for mprobit obs which in some eq got zapped for perfect prediction, remove from all eqs in group
		tempname t
		qui forvalues r=1/$cmp_num_mprobit_groups {
			gen byte `t' = 0 if `if'
			forvalues c=`=cmp_mprobit_group_inds[`r',1]'/`=cmp_mprobit_group_inds[`r',2]' {
				replace `t' = 1 if `if' & _cmp_ind`c'==0
			}
			forvalues c=`=cmp_mprobit_group_inds[`r',1]'/`=cmp_mprobit_group_inds[`r',2]' {
				replace _cmp_ind`c' = 0 if `if' & `t'
			}
			drop `t'
		}
	}

	if "${cmp_cov$parse_L}" == "exchangeable" cap mata st_matrix("`sigvec'", mean(st_matrix("`sigvec'")'))   // can fail if sigvec is empty because all sigs fixed

	tempname Rho rho _sigvec sig t
	forvalues l=1/$parse_L {
		cap mat drop `_sigvec'
		cap mat drop `atanhrho'
		mat `Rho' = I($cmp_d)
		if `l' < $parse_L {
			local cmp_ids `cmp_ids' _cmp_id`l'
			local l_ `l'_
			local level_l "level `l' "
		}
		else {
			local _l
			local level_l
			local _sigvec `sigvec'
		}

		if "${cmp_cov`l'}" == "exchangeable" {
			mat `_sigvec' = 0
			mat `atanhrho' = 0
		}

		quietly forvalues eq=1/$cmp_d {
			if cmp_nonbase_cases[1,`eq'] {
				if "${cmp_rc`l'_`eq'}" != "" & "${cmp_cov`l'_`eq'}" != "exchangeable"  & "${cmp_cov`l'}" != "exchangeable" {
					mat `_sigvec' = nullmat(`_sigvec'), J(1, `:word count ${cmp_rc`l'_`eq'}', `ln'(.1)) // starting value for sd of RC's is .1
				}
				if `l' < $parse_L {
					tempname ebar`eq'
					by `cmp_ids': egen `ebar`eq'' = mean(`e`eq'')
					if ("${cmp_re`l'_`eq'}" != "" | "${cmp_cov`l'_`eq'}" == "exchangeable") & "${cmp_cov`l'}" != "exchangeable" {
						sum `ebar`eq'' `awgtexp'
						scalar `sig' = cond(r(sd), `ln'(r(sd)), `ln'(.1))
						mat `_sigvec' = nullmat(`_sigvec'), `sig'
					}
					replace `e`eq'' = `e`eq'' - `ebar`eq''
				}
				else local ebar`eq' `e`eq''
			}
		}

		forvalues eq1=1/$cmp_d {
			if cmp_nonbase_cases[1,`eq1'] {
				if "${cmp_cov`l'_`eq1'}" == "exchangeable" & cmp_NumEff[`l', `eq1'] mat `atanhrho' = nullmat(`atanhrho'), 0
				forvalues c1=1/`=cmp_NumEff[`l', `eq1']' {
					if "${cmp_cov`l'_`eq1'}" == "unstructured" & `c1' < cmp_NumEff[`l', `eq1'] {
						mat `atanhrho' = nullmat(`atanhrho'), J(1, cmp_NumEff[`l', `eq1'] - `c1', 0)
					}
					if "${cmp_cov`l'}" == "unstructured" {
						forvalues eq2=`=`eq1'+1'/$cmp_d {
							if cmp_fixed_rhos`l'[`eq2',`eq1']==. {
								forvalues c2=1/`=cmp_NumEff[`l', `eq2']' {
									local corrREs = "`:word `c1' of ${cmp_rc`l'_`eq1'}'`:word `c2' of ${cmp_rc`l'_`eq2'}'"==""
									local param `atanh'rho`=cond(`corrREs',"","_`c1'_`c2'")'_`l_'`eq1'`eq2'
									cap corr `ebar`eq1'' `ebar`eq2''
									if r(N) {
										if `corrREs' {
											scalar `rho' = cond(atanh(r(rho))==., 0, atanh(r(rho)))
											mat `Rho'[`eq1',`eq2'] = tanh(`rho')
											mat `Rho'[`eq2',`eq1'] = tanh(`rho')
										}
										else scalar `rho' = 0
										mat `atanhrho' = nullmat(`atanhrho'), cond($cmpSigXform, `rho', tanh(`rho'))
									}
									else {
										if "`drop'" != "" {
											if `c1'==1 & `c2'==1 {
												di as res _n "Samples for `level_l'equations `eq1' and `eq2' do not overlap."
												di as res "`atanh'rho_`l_'`eq1'`eq2' kept in the model because of {cmd:nodrop} option."
												di as res "It cannot be identified, so it must be constrained."
											}
											mat `atanhrho' = nullmat(`atanhrho'), 0
											constraint free
											local initconstraints `initconstraints' `r(free)'
											constraint `r(free)' [`param']_cons
										}
										else {
											if `c1'==1 & `c2'==1  {
												di as res _n "Samples for `level_l'equations `eq1' and `eq2' do not overlap. Removing associated correlation paramter(s) from model."
												mat cmp_fixed_rhos`l'[`eq2',`eq1'] = 0
											}
											local t /`param'
											local auxparams: list auxparams - t
										}
									}
								}
							}
						}
					}
				}
			}
		}
		
		if "${cmp_cov`l'}" == "unstructured" {
			cap mat `Rho' = cholesky(`Rho')
			if _rc { // If initial guess not pos-def, zero out rhos while preserving column labels in case noESTimate will post this
				forvalues c=1/`=colsof(`atanhrho')' {
					mat `atanhrho'[1,`c'] = 0
				}
			}
		}
 		mat `auxparamvec' = `auxparamvec', nullmat(`_sigvec'), nullmat(`atanhrho')
	}

	if colsof(`auxparamvec') > 1 {
		mat `auxparamvec' = `auxparamvec'[1, 2...]
		mat colnames `auxparamvec' = _cons
		mat coleq    `auxparamvec' = `:subinstr local auxparams "/" "", all'
		local ParamsDisplay `ParamsDisplay' `:colfullnames `auxparamvec''
		mat `betavec' = `betavec', `auxparamvec'
	}
	return local ParamsDisplay `ParamsDisplay'
	return local XVarsAll `XVarsAll'
	return matrix b = `betavec'
	return local auxparams `auxparams'
	return local initconstraints `initconstraints'
end

cap program drop Display
program Display, eclass
	version 10.0
	cap version 11.0
	syntax [, Level(real `c(level)') *]
	svyopts noopts diopts, `options'
	mlopts noopts, `noopts' // trigger error if extra options
	local meff meff meft
	local diopts : list diopts - meff
	
	if e(k_gamma) { // make gamma parameters look like regular coefficients
		tempname b V hold
		_estimates hold `hold', copy
		mat `b' = e(b)
		mat `V' = e(V)
		mata _p = st_matrix("e(_p)"); st_matrix("`b'", st_matrix("`b'")[_p]); st_matrix("`V'", st_matrix("`V'")[_p,_p])
		mata mata drop _p
		ereturn repost b=`b' V=`V'
		mat `b' = e(b)
		mat colnames `b' = `e(params)'
		ereturn repost b=`b', rename
		ereturn scalar k_eq  = e(k_eq)  - e(k_gamma)
		ereturn scalar k_aux = e(k_aux) - e(k_gamma)
	}

	if e(L) == 1 {
		if `:word count `e(diparmopt)''/3+`:word count `diopts''<=48+20*(c(version)>10) ml display, level(`level') `diopts' showeq `e(diparmopt)'
																																							 else ml display, level(`level') `diopts' showeq
	}
	else {
		tempname t
		mat `t' = e(num_cuts)' * J(`e(k_dv)', 1, 1)
		ml display, level(`level') `diopts' showeq `=cond(e(sigxform) | `t'[1,1], "", "neq(`e(k_dv)')")' // just displaying cuts causes them to be listed as equations, not aux params
	
		tempname fixed_sigs fixed_rhos NumEff param se z
		scalar `z' = invnormal(.5+`level'/200)
		mat `NumEff' = e(NumEff)

		if e(sigxform) {
			local exp exp
			local ln ln
			local tanh tanh
			local atanh atanh
		}

		di as txt "{hline 36}{c TT}{hline 47}"
		di "Random-effects Parameters           {c |}  Estimate    Std. Err.    [`level'% Conf. Interval]"
		di as txt "{hline 36}{c +}{hline 47}"

		forvalues l=1/`=e(L)-1' {
			local covariance: word `l' of `e(covariance)'
			di "Level: " as res abbrev("`:word `l' of `e(ivars)''", 15) as txt cond("`covariance'"=="exchangeable", " (exchangeable)", "") _col(37) "{c |}" 
			mat `fixed_sigs' = e(fixed_sigs`l')
			mat `fixed_rhos' = e(fixed_rhos`l')
			if "`covariance'" == "exchangeable" {
				local paramname /`ln'sigEx_`l'
				di as txt "    Standard deviations" _col(37) "{c |} " as res %9.0g `exp'(_b[`paramname']) _c
				if _se[`paramname'] di _col(51) %9.0g exp(e(sigxform)*_b[`paramname'])*_se[`paramname'] _col(64) %9.0g `exp'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `exp'(_b[`paramname']+`z'*_se[`paramname'])
					else di as res "  " %9.0g . as txt
				if e(k_dv) > 1 {
					local paramname /`atanh'rhoEx_`l'
					di as txt "    Cross-eq " plural(1+(e(k_dv)>2 | (e(k_dv)==2 & (`NumEff'[`l', 1]>1 | `NumEff'[`l', e(k_dv)]>1))), "correlation") _col(37) "{c |} " as res %9.0g `tanh'(_b[`paramname']) _c
					if _se[`paramname'] di _col(51) %9.0g _se[`paramname']/cosh(e(sigxform)*_b[`paramname'])^2 _col(64) %9.0g `tanh'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `tanh'(_b[`paramname']+`z'*_se[`paramname'])
						else di as res "  " %9.0g . as txt
				}
			}
			forvalues eq1=1/`=e(k_dv)' {
				if `NumEff'[`l', `eq1'] {
					local covariance`eq1': word `l' of `e(covariance`eq1')'
					if e(k_dv)>1 di as txt "  " as res abbrev("`:word `eq1' of `e(eqnames)''", 15) as txt cond("`covariance`eq1''"=="exchangeable", " (exchangeable)", "") _col(37) "{c |}" 
					if  "`covariance'" != "exchangeable" {
						di as txt "    Standard deviations" _col(37) "{c |} " _c as res
						if "`covariance`eq1''"=="exchangeable" {
							local paramname /`ln'sigEx_`l'_`eq1'
							di as res %9.0g `exp'(_b[`paramname']) _c
							if _se[`paramname'] di _col(51) %9.0g exp(e(sigxform)*_b[`paramname'])*_se[`paramname'] _col(64) %9.0g `exp'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `exp'(_b[`paramname']+`z'*_se[`paramname'])
								else di "  " %9.0g .
						}
						else {
							di as txt
							forvalues c=1/`=`NumEff'[`l', `eq1']' {
								local t = abbrev("`:word `c' of `e(EffNames`l'_`eq1')''", 22)
								di as txt "      `t'"  _col(37) "{c |} " _c
								if `fixed_sigs'[1,`eq1'] == . {
									local paramname /`ln'sig`=cond("`t'"=="_cons","","_`c'")'_`l'_`eq1'
									di as res %9.0g `exp'(_b[`paramname']) _c
									if _se[`paramname'] di _col(51) %9.0g exp(e(sigxform)*_b[`paramname'])*_se[`paramname'] _col(64) %9.0g `exp'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `exp'(_b[`paramname']+`z'*_se[`paramname'])
										else di "  " %9.0g . as txt
								}
								else di %9.0g `fixed_sigs'[1,`eq1'] as txt "  (constrained)"
							}
						}
					}
					if `NumEff'[`l', `eq1'] > 1 & "`covariance`eq1''"!="independent" {
						di as txt "    Intra-eq " plural(`NumEff'[`l', `eq1']-1, "correlation") _col(37) "{c |} " _c
						if "`covariance`eq1''"=="exchangeable" {
							local paramname /`atanh'rhoEx_`l'_`eq1'
							di %9.0g `tanh'(_b[`paramname']) _c
							if _se[`paramname'] di _col(51) %9.0g _se[`paramname']/cosh(e(sigxform)*_b[`paramname'])^2 _col(64) %9.0g `tanh'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `tanh'(_b[`paramname']+`z'*_se[`paramname'])
								else di "  " %9.0g .
						}
						else {
							di
							forvalues c1=1/`=`NumEff'[`l', `eq1']' {
								forvalues c2=`=`c1'+1'/`=`NumEff'[`l', `eq1']' {
									local t1 = abbrev("`:word `c1' of `e(EffNames`l'_`eq1')''", 15)
									local t2 = abbrev("`:word `c2' of `e(EffNames`l'_`eq1')''", 15)
									local paramname /`atanh'rho_`c1'_	`c2'_`l'_`eq1'
									di as txt "      `t1'" _col(20) "`t2'" _col(37) "{c |} " as res %9.0g `tanh'(_b[`paramname']) _c
									if _se[`paramname'] di _col(51) %9.0g _se[`paramname']/cosh(e(sigxform)*_b[`paramname'])^2 _col(64) %9.0g `tanh'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `tanh'(_b[`paramname']+`z'*_se[`paramname'])
										else di "  " %9.0g .
								}
							}
						}
					}
				}
			}
			if "`:word `l' of `e(covariance)''" == "unstructured" & e(k_dv) > 1 {
				local needheader 1
				forvalues eq1=1/`e(k_dv)' {
					forvalues eq2=`=`eq1'+1'/`e(k_dv)' {
						if `fixed_rhos'[`eq2',`eq1'] == . & `=`NumEff'[`l', `eq1']' & `=`NumEff'[`l', `eq2']' {
							if `needheader' di as txt " Cross-eq " plural(1+(e(k_dv)>2 | (e(k_dv)==2 & (`NumEff'[`l', 1]>1 | `NumEff'[`l', e(k_dv)]>1))), "correlation") _col(37) "{c |} "
							local needheader 0
							di as txt "  " as res abbrev("`:word `eq1' of `e(eqnames)''", 15) _col(19) abbrev("`:word `eq2' of `e(eqnames)''", 15) as txt _col(37) "{c |}"
							forvalues c1=1/`=`NumEff'[`l', `eq1']' {
								forvalues c2=1/`=`NumEff'[`l', `eq2']' {
									local t1 = abbrev("`:word `c1' of `e(EffNames`l'_`eq1')''", 15)
									local t2 = abbrev("`:word `c2' of `e(EffNames`l'_`eq2')''", 15)
									local paramname /`atanh'rho`=cond("`t1'`t2'"=="_cons_cons", "", "_`c1'_`c2'")'_`l'_`eq1'`eq2'
									di as txt "    `t1'" _col(21) "`t2'" _col(37) "{c |} " as res %9.0g `tanh'(_b[`paramname']) _c
									if _se[`paramname'] di _col(51) %9.0g _se[`paramname']/cosh(e(sigxform)*_b[`paramname'])^2 _col(64) %9.0g `tanh'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `tanh'(_b[`paramname']+`z'*_se[`paramname'])
										else di "  " %9.0g .
								}
							}
						}
					}
				}
			}
			di as txt "{hline 36}{c +}{hline 47}"
		}

		mat `fixed_sigs' = e(fixed_sigs`e(L)')
		mat `fixed_rhos' = e(fixed_rhos`e(L)')
		di "Level: " as res "Residuals" _col(37) as txt "{c |}"
		di " Standard " plural(e(k_dv), "deviation") _col(37) "{c |} " as res  _c
		if "`:word `e(L)' of `e(covariance)''" == "exchangeable" {
			local paramname /`ln'sigEx
			di %9.0g `exp'(_b[`paramname']) _c
			if _se[`paramname'] di _col(51) as res %9.0g exp(e(sigxform)*_b[`paramname'])*_se[`paramname'] _col(64) %9.0g `exp'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `exp'(_b[`paramname']+`z'*_se[`paramname'])
				else di "  " %9.0g . as txt
			if e(k_dv) > 1 {
				di as txt " Cross-eq " plural(e(k_dv)-1, "correlation") _col(37) "{c |} " _c
				local paramname /`atanh'rhoEx
				di as res %9.0g `tanh'(_b[`paramname']) _c
				if _se[`paramname'] di _col(51) as res %9.0g _se[`paramname']/cosh(e(sigxform)*_b[`paramname'])^2 _col(64) %9.0g `tanh'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `tanh'(_b[`paramname']+`z'*_se[`paramname'])
					else di "  " %9.0g .
			}
		}
		else {
			if e(k_dv)>1 di
			forvalues eq1=1/`=e(k_dv)' {
				if e(k_dv)>1 di as txt "  " as res abbrev("`:word `eq1' of `e(eqnames)''", 24) as txt _col(37) "{c |} " _c
				if `fixed_sigs'[1,`eq1'] == . {
					local paramname /`ln'sig_`eq1'
					di as res %9.0g `exp'(_b[`paramname']) _c
					if _se[`paramname'] di _col(51) %9.0g exp(e(sigxform)*_b[`paramname'])*_se[`paramname'] _col(64) %9.0g `exp'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `exp'(_b[`paramname']+`z'*_se[`paramname'])
						else di "  " %9.0g .
				}
				else di as res %9.0g `fixed_sigs'[1,`eq1'] as txt "  (constrained)"
			}
			if e(k_dv) > 1 & "`:word `e(L)' of `e(covariance)''" == "unstructured" {
				local needheader 1
				forvalues eq1=1/`e(k_dv)' {
					forvalues eq2=`=`eq1'+1'/`e(k_dv)' {
						if `fixed_rhos'[`eq2',`eq1'] == . {
							if `needheader' di as txt " Cross-eq " plural(e(k_dv)-1,"correlation") _col(37) "{c |} "
							local needheader 0
							di as txt "  " as res abbrev("`:word `eq1' of `e(eqnames)''", 15) _col(19) as res abbrev("`:word `eq2' of `e(eqnames)''", 15) as txt _col(37) "{c |} " _c
							local paramname /`atanh'rho_`eq1'`eq2'
							di as res %9.0g `tanh'(_b[`paramname']) _c
							if _se[`paramname'] di _col(51) %9.0g _se[`paramname']/cosh(e(sigxform)*_b[`paramname'])^2 _col(64) %9.0g `tanh'(_b[`paramname']-`z'*_se[`paramname']) _col(76) %9.0g `tanh'(_b[`paramname']+`z'*_se[`paramname'])
								else di "  " %9.0g .
						}
					}
				}
			}
		}
		di as txt "{hline 36}{c BT}{hline 47}"
	}
	if e(k_gamma) _estimates unhold `hold'
end

cap program drop CheckCondition
program define CheckCondition
	version 10.0
	cap version 11.0
	if "`1'" != "" {
		syntax varlist(ts `=cond(c(version)==11,"fv","")') [aw iw]
		tempname XX c
		`=cond(c(version)==11,"fv","ts")'revar `varlist'
		local varlist `r(varlist)'
		qui mat accum `XX' = `varlist' [`weight'`exp'] if e(sample), nocons
		forvalues i=1/`=colsof(`XX')' {
			if `XX'[`i',`i'] local xvars `xvars' `:word `i' of `varlist''
		}
		if "`xvars'" != "" {
			qui mat accum `XX' = `xvars' [`weight'`exp'] if e(sample), nocons
			mata st_numscalar("`c'", cond(corr(st_matrix("`XX'"))))
			if `c' > 20 { // threshhold from Greene (2000, p. 40)
				di _n as res "Warning: regressor matrix for " as txt "`e(depvar)'" as res " equation appears ill-conditioned. (Condition number = " `c' ".)"
				di "This might prevent convergence. If it does, and if you have not done so already, you may need to remove nearly"
				di "collinear regressors to achieve convergence. Or you may need to add a {opt nrtol:erance(#)} or {opt nonrtol:erance} option to the command line."
				di "See {help cmp##tips:cmp tips}."
			}
		}
	}
end

cap program drop cmp_error
program define cmp_error
	version 10.0
	noi di as err `"`2'"'
	cmp_clear
	exit `1'
end

* Version history
* 6.4.2 Warn on non-prime draw counts. In factor-variable Stata versions (>10) omit vars that have 0 standard error in single-equation fits.
*       Dropped 5.4.4's "nonrtolerance tolerance(0.1)" for safety. E.g., loose tobit fit may misjudge which variables ought to be dropped (indicated by singular variance).
*       Fixed 6.4.0 bug: bounds() option broke code in interactive mode. Moved from ml model to ml max command.
* 6.4.1 Use traditional product rule instead of "sparse" grid for 2-D problems. Eliminate small-sample correction to C in Naylor-Smith adaptation that wasn't reflected in AdaptiveShift.
* 6.4.0 Added undocumented noSIGXform option to parameterize with sig's and rho's instead of lnsig's and atanhrho's.
*       Fixed labeling on non-mprobit eqs in mixed mprobit system using short mprobit syntax
*       Label e(b) and e(V) with true model so that scores can be computed, e.g., as requested for svy
* 6.3.2 Restored display of sigs and rhos in non-hierarchical models (bug).
* 6.3.1 Made sure to set _GammaInds and set _interactive=1 in cmp...predict. Was preventing score prediction, including during svy:.
* 6.3.0 Added adaptive quadrature, by default for random effects/coefficients models. Disabled 6.0.3's setting of favorspeed.
* 6.2.0 Added sparse-grid quadrature as default replacement for simulation-based modelling of random effects and coefficients.
* 6.1.0 Added support for hierarchical Gamma models. Fixed non-convergence in hierarchical mprobit models & models with >2 levels. Cleaned up display of hierarchical results.
* 6.0.4 Tightened Mata code. Fixed Gamma-related bug in score construction in cmp_d1 for Stata 10.
*       Fixed bug in binormalGenz() introduced in 6.0.3. Fixed new bug in handling mprobit eqs that are 0 for some obs, 6 for others
* 6.0.3 Fixed bug in Gamma models with constraints in constants-only fit.
*       Zero out Gamma scores for obs in which a Gamma-including eq is dropped.
*       Switch to favorspeed to avoid apparent bug in ml.
* 6.0.2 Fixed Gamma models messing up displayed parameter names when constraints used 
* 6.0.1 Fixed handling of constants-only fit in Gamma models
* 6.0.0 Allowed references to (latent) linear dep vars, with full simultaneity. Eliminated serious bug in ThetaSign computation in models with more than 2 levels.
* 5.4.5 In InitSearch, for mprobit obs which in some eq get dropped for perfect prediction, remove from all eqs in group
* 5.4.4 Fixed bugs in InitSearch in recognizing factor variables dropped by probit or _rmcoll. Made cmp_full_model omit o. FV's from fvrevar'd lists.
*       Stopped rerunning initial fits when variables dropped in Stata 11 or later. Added "nonrtolerance tolerance(0.1)" for faster, coarser initial fits.
* 5.4.3 Fixed Mata bugs in multi-equation truncation models.
* 5.4.2 Fixed bug causing e(wexp)="=" when no weights used.
* 5.4.1 Fixed typo in code for interactive mode. Updated cmp_d1 and cmp_lf.
* 5.4.0 Made truncation available in all models except multinomial and rank-ordered probit. Thanks to Tamas Bartus for the suggestion.
*       Added custom display of RE and RC results.
*       Interpret "exchangeable" to mean all sigs in a group, not just rhos, the same.
*       Went public with random coefficients
*       Fixed misc bugs
* 5.3.1 Fixed InitSearch bug for eqs with RCs but no RE.
*       Implemented proper dSigdParams construction for RCs.
*       Added use of undocumented ADDCONS feature with buildfvinfo.
* 5.3.0 Added random coefficients (undocumented, beta). Fixed bug in redraws(, anti).
* 5.2.6 Added informative error message if i. variables get different bases in different eqs
* 5.2.5 Prevented crash when two eqs have same name, by suffixing with eq number
*       Fixed bug in constants-only InitSearch when dropping rhos for non-overlapping eqs
* 5.2.4 Dropped e(diparmopt) from call to ml display if that exceeds maximum # of options for syntax command
* 5.2.3 Fixed bug causing crash in mixed censored-uncensored models.
*       Used ereturn, repost to establish correct esample() before using buildfvinfo option. Prevented revar'ing when constraints used.
* 5.2.2 Assured intreg residuals in InitSearch are never missing even if bounds are
* 5.2.1 Changed _rmdcoll to _rmcoll exept for regtype==1
* 5.2.0 Added lnl option to predict
* 5.1.2 Prevented crash when adjustprobitsample restricts sample on only eq that applies to obs, thus restricting sample
*       Prevented crash when model with oprobit has some obs with no oprobit (or otherwise censored) eqs for some obs
* 5.1.1 Prevented crash when diparm tries to display rho version of a dropped atanhrho
* 5.1.0 Added covariance() option
* 5.0.0 Added random effects, iia suboption. Dropped pseudod2.
* 4.0.4 Minor changes in handling of rho's for non-overlapping eqs in constant-only model
* 4.0.3 Added check after _rmdcoll to remove "o." vars in varlist--artifact of factor variable support
* 4.0.2 Tweaked InitSearch tobit code introduced in 3.5.2 for factor var compatibility. No "version 10:" now.
* 4.0.1 Code refinements
* 4.0.0 Added rank-ordered probit support.
* 3.9.3 In InitSearch, eliminated subscripting matrices with "#1" in Stata-provided code fragment for compatibility with Stata <11.
* 3.9.2 Suppressed call to test introduced in 3.9.1 if LR test used 
* 3.9.1 Fixed bugs in handling factor variables with help of Stata Corp. 
*       For intreg, made InitSearch include samples where indicator==1, not just 7
* 3.9.0 Added factor variable support. Thanks to Tamas Bartus for inspiration.
*       Use pseudod2 in Stata version 11.1 or earlier. Code to check version is version dependent! date() takes "dmy" in some versions, "DMY" in others
* 3.8.6 Fixed typo in 3.8.5. Suppressed "equation #1 assumed" in predict if there is only 1 equation
* 3.8.5 Use pseudod2 if Stata born before 11 feb 2010
* 3.8.4 Fixed bug so pseudod2 will call ml to be run under version 9.2 so that cluster() is accepted
*       Fixed bug introduced in 3.6.8--code for vecmultinormal() call for truncregs not properly updated
* 3.8.3 Fixed bug in InitSearch preventing dropping of last regressor from an eq if no SE in preliminary regression and nocons
*       Fixed bug in line determinining version to use for ml. (local a=c(version) can return 9.199999999)
* 3.8.2 Added lnl() option to save observation-level log likelihood. Fixed 3.8.0 bug.
* 3.8.1 Use arguments instead of Stata locals to pass to cmp_lnL() variable names for scores and likelihood
*       Changed required ghk2() version to 1.3.1 because of bug fix in latter
* 3.8.0 Incorporated use of lf1 method by default in Stata 11 and later
* 3.7.0 Fixed minor bugs in InitSearch in handling case of no RHS vars
*       Fixed bug restricting whole-system sample to those of non-as mprobit eqs
* 3.6.9 Got rid of i loop in vecbinormal() for speed
* 3.6.8 Made vecmultinormal() return value and scores of log instead of level. Tightened functions that call it.
* 3.6.7 In cmp_d2, changed constant in h formula from 1e-4 to 1e-5.
*       Introduced normal2() to calculate normal(U)-normal(L) more precisely.
* 3.6.6 Added check in Parse for missing "="
* 3.6.5 Fixed 3.6.4 bug: predict didn't handle SigScoreInds properly when it was empty
* 3.6.4 Fixed bug: code for dropping rhos of non-overlapping eqs broken by 3.5.0.
*       Fixed bug: changed `0' to `"`0'"' in call to cmp_full_model in case command line contains quotes
*       Fixed bug: lack of SE for _cons in InitSearch probit fit didn't cause its coef to be dropped from initial fit vector
* 3.6.3 Added check to cmp_d2 for lnf = 0 after cmp_lnL call
* 3.6.2 Reworked `quietly' 3.5.5 work-around. Put bracketed code in DoInitSearch
* 3.6.1 Fixed bug in 3.6.0
* 3.6.0 Added noestimate option, copying gllamm. Thanks to Stas Kolenikov for suggestion.
* 3.5.6 Made mi-friendly
* 3.5.5 Inserted line break in "`quietly' {" to avoid Stata syntactic pecadillo.
*       If inital guess for Sigma is not positive definite, InitSearch makes it diagonal
* 3.5.4 Allowed reals for level()
* 3.5.3 Added warning about initial single-equation fits deviating from specification.
* 3.5.2 For weighted regressions, InitSearch now uses iweights throughout, except for -tobit- in Stata 9.2, where it must use aweights.
*       This improves starting point. -tobit- does aweights differently from -ml-. (see [R] intreg)
* 3.5.1 Added "did you forget to type cmp setup?"
* 3.5.0 Reorganized to leave no variables behind. Now works with -est (re)store- and -suest-. Dropped cleanup/clear subcommand.
* 3.4.6 Removed bug causing attempt to store potentially non-numeric value labels in e(cat) for oprobit eq
* 3.4.5 Added warning that with GHK changing observation order changes results.
* 3.4.4 Added nopreserve to ml model command in interactive mode
* 3.4.3 Added e(cmdline)
* 3.4.2 Added clear as synonym for cleanup
* 3.4.1 Tightened vecbinormal(), neg_half_E_Dinvsym_E(), and dPhi_dpE_dSig(). Took over model parsing and changed truncreg syntax
* 3.4.0 Added truncreg equation type
* 3.3.3 Run InitSearch even when init() specified in order to consistently perform and report various specification checks. Thanks to Misha Bontch-Osmolovski.
* 3.3.2 Made it use distinct GHK draws for each block of identically censored observations, via new ghk2() s argument
* 3.3.1 Added warning about Stata 9 cluster() behavior with missing values in regressors. Thanks to Misha Bontch-Osmolovski.
* 3.3.0 Added ghk2version check
* 3.2.9 Switched from invsym() to cholinv(). Fixed bug in cmp_p affecting predicted probability after ordered probit for second-highest outcome
* 3.2.8 Added -missing- option to ml model call in interactive model to prevent sample shrinkage
* 3.2.7 Fixed loss of sample marker caused by preserve/restore in Estimate
* 3.2.6 Fixed bug causing crash with use of init()
* 3.2.5 Fixed robust/cluster handling incompatibility between Estimate and Stata 9
* 3.2.4 Changed =r(varlist) to `r(varlist)' after tsrevar so it doesn't treat macros as strings
* 3.2.3 Fixed bugs affecting mprobit score and Hessian computations. mprobit converges much better. Fixed misc bugs.
* 3.2.2 Used tsrevar and preserve/keep/restore to cut down data set for speed in non-interactive mode. Created Estimate subroutine.
*       For non-overlapping samples removed rho_ij from model rather than keeping and constraining to 0, for speed.
* 3.2.1 Added e() and ystar() options to cmp_p
* 3.2.0 Added interval regression type. Replaced minfloat() and maxfloat() for cuts with "."
* 3.1.1 Fixed bug preventing use of user-supplied equation names. Added "_cons" after ":" in _b[] in cmp_p
* 3.1.0 Added psuedo-d2 evaluator as default
* 3.0.3 Fixed call to _ghk2_2d() had dE and dF swapped
* 3.0.2 Made sure empty score matrix still created for cuts in obs with no oprobit eq
* 3.0.1 Fixed bug in determining which cut #s are relevant for which obs when set of oprobit eqs varies
* 3.0.0 Added multinomial probit support. Added lf evaluator. Switched to ghk2(). Added progressive sampling.
* 2.1.0 Fixed bug causing constants-only model to be unweighted
* 2.0.9 Replaced call to symeigenvalues() with one to cholesky() for surprisingly large speed gain.
* 2.0.8 Tightened Mata code. Put missing qui in InitSearch.
* 2.0.7 Fixed bugs in Mata code for ordered probit. Wasn't working if # of o-probit eq varied by obs.
* 2.0.6 Fixed small bug in cmp_p
* 2.0.5 Fixed 2.0.4 bug affecting whether constant included in an ordered probit equation
* 2.0.4 Fixed 2.0.2 bug. Estimation restricted to union of samples for each eq. Was the intersection before 2.0.2. Was set by if/in only in 2.0.2.
* 2.0.3 When using dep var as eq name, remove "." from ts ops
* 2.0.2 Changed response to missing obs. If indicator>0 and obs missing, no longer set touse=0 for all eq--just set indicator=0.
* 2.0.1 Changed e(indicators) to contents of indicators() option. (e(k_dv) holds # of indicators.)
*       Fixed bug in dPhi_dpE_dSig() for ordered-probit case.
* 2.0.0 Added ordered probit and beefed up predict.
* 1.4.1 Added real generation of residuals after probit in InitSearch, so 1.3.1 code doesn't think probit samples overlap no others
* 1.4.0 Added init() option for manual control of initial values
* 1.3.1 Added rho constraints to handle equations with non-overlapping subsamples
* 1.3.0 Added plot feature. Added _rmdcoll check to InitSearch.
* 1.2.8 Fixed typo ("`drop'"") introduced in 1.2.7
* 1.2.7 Turned ghkanti option from an integer to a present/absent macro. Added nodrop option.
* 1.2.6 Changed e(indicators) from macro to scalar
* 1.2.5 Added return macros e(Ni) with equation-specific sample sizes. Prevented errors if cmp cleanup run unnecessarily.
*       Fixed bugs in handling of ts ops.
*       Prevented it from dropping all regressors if an initial 1-equation tobit fails to converge.
* 1.2.4 In interactive mode, moved mlopts from ml model to ml max command, where they belong.
*       Adjusted 1.2.3 fix: iweights for probit and aweight for tobit and regress.
*       Added warning for ill-conditioned regressor matrix.
* 1.2.3 Use aweights instead of pweights, if requested, in InitSearch since Stata 9.2 tobit doesn't accept pweights, and for speed
* 1.2.2 Fixed bug in InitSearch causing it to drop observations with missing even when the missings are in variables/equationsare marked as out
* 1.2.1 Changed "version 9.2" to "cap version 10" in cmp_ll so callersversion() returns right value.
* 1.2.0 Made it work in Stata 9.2
* 1.1.2 Added noclear to ml command line in interactive mode
* 1.1.1 Prevented repeated display of ghk notification in interactive mode
* 1.1.0 Added interactive option
* 1.0.2 Fixed predict statement after bicensored tobit in InitSearch
* 1.0.1 Minor changes
