*! cmp 6.4.0 5 July 2013
*! Copyright David Roodman 2007-13. May be distributed free.
cap program drop cmp_p
program define cmp_p
	version 10.0
	cap version 11.0

	syntax anything [if] [in], [EQuation(string) Outcome(string) NOOFFset pr REsiduals lnl SCores Ystar(string) e(string) REDucedform *]
	if `: word count `pr' `residuals' `scores' `ystar' `e' `lnl'' > 1 {
		di as err "Only one statistic allowed per {cmd:predict} call."
		exit 198
	}

	local options `options' `scores'
	marksample touse, novarlist
	
	if "`scores'"=="" {
		tempname b
		mat `b' = e(b)
		mat `b' = `b'[1,1..`=e(k)-e(k_aux)']
	}
	else qui replace `touse' = `touse' & e(sample)

	if `"`outcome'"'!="" local pr pr
	_score_spec `anything', equation(`equation') b(`b')
	if "`s(eqspec)'"=="#1" & `"`equation'`lnl'"'=="" & e(k_dv)>1 di as txt "(equation #1 assumed)"

	local vartype : word 1 of `s(typlist)'
	local varlist `s(varlist)'

	if "`lnl'" != "" {
		_stubstar2names `anything', nvars(1)
		local vartype `s(typlist)'
		local varname `s(varlist)'
		quietly {
			gen `vartype' `varname' = .
			`e(cmdline)' predict // run cmp front end to reconstruct necessary _cmp_* variables
			tempname hold
			_est hold `hold', copy
			ml model `e(model)' if e(sample) & `touse'
			_est unhold `hold'
			if "`e(user)'" == "cmp_lf" {
				tempvar t
				_stubstar2names `t'*, nvars(`e(k_eq)')
				tokenize `s(varlist)'
				forvalues i=1/`e(k_eq)' {
					Predict double ``i'' if e(sample) & `touse', eq(#`i') `reducedform'
				}
				cmp_lf `varname' `*'
			}
			else {
				tempname b
				mat `b' = e(b)
				if "`e(user)'" == "cmp_lf1" {
					cmp_lf1 0 `b' `varname'
				}
				else {
					tempname t
					cmp_d1 0 `b' `t'
					replace `varname' = _cmp_lnfi if e(sample) & `touse'
				}
			}
			cmp_clear
		}
		exit
	}
	
	if `"`options'`pr'`residuals'`ystar'`e'"' == "" di as txt "(option xb assumed; fitted values)"

	tempname num_cuts cat
	mat `cat' = e(cat)
	mat `num_cuts' = e(num_cuts)

	tokenize `varlist'
	for`=cond("`s(eqspec)'"=="", "values eq=1/`e(k_`=cond("`scores'"=="", "dv", "eq")')'", "each eq in `=subinstr("`s(eqspec)'","#","",.)'")' {
		local depvar : word `eq' of `e(depvar)'
		if `"`pr'`residuals'`ystar'`e'"' == "" {
			quietly `e(cmdline)' predict // run cmp front end to reconstruct necessary _cmp_* variables
			Predict `vartype' `1' if `touse', `options' eq(#`eq') `offset' `reducedform'
			cmp_clear
		}
		else {
			tempvar xb
			Predict double `xb' if `touse', `options' eq(#`eq') `offset' `reducedform'
		
			if "`residuals'" != "" {
				gen `vartype' `1' = `depvar' - `xb' if `touse'
				label var `1' Residuals
			}
			else if `"`ystar'`e'"' != "" {
				local ll : word 1 of `ystar'`e'
				local ul : word 2 of `ystar'`e'
				cap confirm var `ll'
				local lmissing = _rc==0 & `ll'>=.
				cap confirm var `ul'
				local umissing = _rc==0 & `ul'>=.
				local title = cond(`lmissing' & `umissing', "", "|`=cond(`lmissing', "", "`ll'<")'`depvar'`=cond(`umissing', "", "<`ul'")'")
				cap local sig = exp([lnsig_`eq']_cons)
				if _rc local sig 1
				tempvar L U phiL phiU PhiL PhiU
				qui {
					gen double `L' = (`ll'-`xb')/`sig' if `touse'
					gen double `U' = (`ul'-`xb')/`sig' if `touse'
					gen double `phiL' = cond(`ll'==., 0, normalden(`L')) if `touse'
					gen double `phiU' = cond(`ul'==., 0, normalden(`U')) if `touse'
					gen double `PhiL' = cond(`ll'==., 0, normal   (`L')) if `touse'
					gen double `PhiU' = cond(`ul'==., 1, normal   (`U')) if `touse'
				}
				if `"`e'"' != "" {
					gen `vartype' `1' = `xb' - `sig' * (`phiU'-`phiL')/(`PhiU'-`PhiL') if `touse'
					label var `1' "E(`depvar'`title')"
				}
				else {
					gen `vartype' `1' = (`PhiU'-`PhiL')*`xb'-`sig'*(`phiU'-`phiL')+cond(`ll'==.,0,`PhiL'*`ll')+cond(`ul'==.,0,(1-`PhiU')*`ul') if `touse'
					label var `1' "E(`depvar'*`title')"
				}
			}
			else if "`pr'" != "" {
				local num_cats = `num_cuts'[`eq',1] + 1
				if `num_cats' > 1 {
					if `"`outcome'"' == "" {
						_stubstar2names `1'_*, nvars(`num_cats') outcome
						forvalues outno=1/`num_cats' {
							gen `vartype' `1'_`outno' = `=cond(`outno'<=`num_cuts'[`eq',1], "normal([cut_`eq'_`outno']_cons - `xb')", "1")' ///
												    - `=cond(`outno'>1, "normal([cut_`eq'_`=`outno'-1']_cons - `xb')", "0")'
							label var `1'_`outno' "Pr(`depvar'=`=`cat'[`eq', `outno']')"
						}
					}
					else {
						if substr(`"`outcome'"', 1, 1) == "#" {
							local outcome = substr(`"`outcome'"', 2, .)
							if `outcome' > `num_cats' {
								di as err `"There is no outcome #`outcome'. There are only `num_cats' outcomes for equation #`eq'."'
								exit 111
							}
						}
						else {
							local i 1
							while `i' <= `num_cats' & `cat'[`eq', `i']!=`outcome' {
								local ++i
							}
							if `i' > `num_cats' {
								di as error `"Outcome `outcome' not found in equation `eq'. outcome() must either be a value of `depvar' or #1, #2, ..."'
								exit 111
							}
							local outcome `i'
						}
						gen `vartype' `1' = `=cond(`outcome'<=`num_cuts'[`eq',1], "normal([cut_`eq'_`outcome']_cons - `xb')", "1")' ///
											- `=cond(`outcome'>1, "normal([cut_`eq'_`=`outcome'-1']_cons - `xb')", "0")'
						label var `1' "Pr(`depvar'=`=`cat'[`eq', `outcome']')"
					}
				}
				else if `"`outcome'"' == "" {
					gen `vartype' `1' = normal(`xb')
					label var `1' "Pr(`depvar')"
				}
				else {
					di as err "Equation #`eq' is not ordered probit. outcome() is not allowed."
					exit 197
				}
			}
		}
		macro shift
	}
end

cap program drop Predict
program Predict, eclass
	syntax anything [if], eq(string) [reducedform *]
	
	local hasGamma = e(k_gamma) & "`options'"!="scores"
	if `hasGamma' {
		tempname b V N hold
		tempvar sample
		local reducedform = "`reducedform'"!=""
		if `reducedform'==0 {
			mat `b' = e(b)
			mat colnames `b' = `e(params)'
			mat `b' = `b'[1,`"`:word `=substr("`eq'", 2, .)' of `e(eqnames)'':"']
			local reducedform = strpos("`:colnames `b''", "#")
		}
		if `reducedform' {
			mat `b' = e(br)
			mat `V' = e(Vr)
			scalar `N' = e(N)
			gen byte `sample' = e(sample)
			_estimates hold `hold'
			eret post `b' `V', obs(`=`N'') esample(`sample')
		}
		else {
			_estimates hold `hold', copy
			mat `b' = e(b)
			mat `V' = e(V)
			mata _p = st_matrix("e(_p)"); st_matrix("`b'", st_matrix("`b'")[_p]); st_matrix("`V'", st_matrix("`V'")[_p,_p])
			mata mata drop _p
			ereturn repost b=`b' V=`V'
			mat `b' = e(b)
			mat colnames `b' = `e(params)'
			ereturn repost b=`b', rename
		}
	}
	
	qui ml_p `anything' `if', `options' equation(`eq')
	
	cap _estimates unhold `hold'
end
